﻿using UnityEditor;
using UnityEngine;

namespace Playground.EditorInternal
{
    [CustomPropertyDrawer(typeof(Condition), useForChildren: true)]
    public class ConditionDrawer : PropertyDrawer
    {
        protected static GUIContent _ConditionCompleteContent;
        protected static GUIContent ConditionCompleteContent
        {
            get
            {
                if(_ConditionCompleteContent == null)
                {
                    _ConditionCompleteContent = new GUIContent(Resources.Load<Texture2D>("Complete"));
                }
                return _ConditionCompleteContent;
            }
        }

        protected static GUIContent _ConditionIncompleteContent;
        protected static GUIContent ConditionIncompleteContent
        {
            get
            {
                if (_ConditionIncompleteContent == null)
                {
                    _ConditionIncompleteContent = new GUIContent(Resources.Load<Texture2D>("Fatal"));
                }
                return _ConditionIncompleteContent;
            }
        }

        public override float GetPropertyHeight( SerializedProperty property, GUIContent label )
        {
            return EditorGUIUtility.singleLineHeight * 2 + 4;
        }
        public override void OnGUI( Rect position, SerializedProperty property, GUIContent label )
        {
            GUI.Box(position, GUIContent.none);
            Condition condition = (property.objectReferenceValue as Condition);
            if(!condition)
            {
                EditorGUI.LabelField(position, "NULL");
                return;
            }
            position = new Rect(position) { x = position.x + 2, y = position.y + 2, width = position.width - 4, height = position.height - 4 };
            Rect iconPos = new Rect(position) { width = position.height };
            Rect labelPos = new Rect(position) { width = position.width - iconPos.width, x = iconPos.x + iconPos.width + 8 };

            GUI.Label(iconPos, condition.Evaluate() ? ConditionCompleteContent : ConditionIncompleteContent);
        
            GUI.Label(position, condition.name, EditorStyles.centeredGreyMiniLabel);


        }
    }

    [CustomPropertyDrawer(typeof(ConditionGroup), useForChildren: true)]
    public class ConditionGroupDrawer : ConditionDrawer
    {
        [SerializeField]
        private bool _expanded;
        public override float GetPropertyHeight(SerializedProperty property, GUIContent label)
        {
            
            SerializedProperty it = property.Copy();
            float totalHeight = EditorGUIUtility.singleLineHeight;
            if(_expanded)
            {
                while (it.NextVisible(true))
                {
                    totalHeight += EditorGUI.GetPropertyHeight(it, label, true) + EditorGUIUtility.standardVerticalSpacing;
                }
            }
            return totalHeight;
        }
        public override void OnGUI(Rect position, SerializedProperty property, GUIContent label)
        {
            Condition condition = (property.objectReferenceValue as Condition);
            _expanded = EditorGUI.BeginFoldoutHeaderGroup(position, _expanded, condition?.GetType().Name ?? "NULL");
            if(_expanded)
            {
                GUI.Box(position, GUIContent.none);
                if (!condition)
                {
                    EditorGUI.LabelField(position, "NULL");
                    return;
                }
                position = new Rect(position) { x = position.x + 2, y = position.y + 2, width = position.width - 4, height = position.height - 4 };
                Rect iconPos = new Rect(position) { width = position.height };
                Rect labelPos = new Rect(position) { width = position.width - iconPos.width, x = iconPos.x + iconPos.width + 8 };

                GUI.Label(iconPos, condition.Evaluate() ? ConditionCompleteContent : ConditionIncompleteContent);

                GUI.Label(position, condition.name, EditorStyles.centeredGreyMiniLabel);

            }
            EditorGUI.EndFoldoutHeaderGroup();
        }
    }
}