﻿using System;
using UnityEngine;
using System.Collections.Generic;
using System.Linq;
namespace Playground.Internal
{
    using UI;


    public static class ConditionTypes
    {
        public static readonly List<Type> GroupTypes = new List<Type>();
        public static readonly List<Type> Types = new List<Type>();
        public static readonly Dictionary<Type, Type> Editors = new Dictionary<Type, Type>();


        static ConditionTypes()
        {
            EnlistConditionTypes();
        }

        /// <summary>
        /// 
        /// </summary>
        private static void EnlistConditionTypes()
        {
            foreach (Type conditionType in typeof(Condition).Assembly.GetTypes().Where(t => !t.IsAbstract && t.IsSubclassOf(typeof(Condition))))
            {
                Debug.Log($"Adding Condition \"{conditionType.Name}\" to list of Condition types...");
                if(conditionType.IsSubclassOf(typeof(ConditionGroup)))
                    GroupTypes.Add(conditionType);
                else 
                    Types.Add(conditionType);
            }
            foreach (Type editorClass in typeof(Condition).Assembly.GetTypes().Where(t => !t.IsAbstract && typeof(IConditionEditor).IsAssignableFrom(t)))
            {
                var targetingAttrib = (ConditionEditorAttribute)editorClass.GetCustomAttributes(typeof(ConditionEditorAttribute), true).FirstOrDefault();

                if (targetingAttrib != null)
                    Editors.Add(targetingAttrib.Target, editorClass);
                else
                    Debug.LogWarning($"{editorClass} Has not annotated condition target!");
            }
        }

        /// <summary>
        /// Create an editor for a given condition
        /// </summary>
        /// <param name="condition">the condition we want to open for editing through the editor</param>
        /// <returns></returns>
        internal static IConditionEditor CreateConditionEditorObject(Condition condition)
        {
            if (Editors.TryGetValue(condition.GetType(), out var editorType))
            {
                var conditionEditor = (IConditionEditor)new GameObject($"{editorType.Name}").AddComponent(editorType);
                conditionEditor.Initialize(condition);
                return conditionEditor;
            }
            else Debug.LogError($"There are no registered editor types for {condition}!");
            //TODO default editor?
            return null;
        }
    }
}
