﻿using System.Collections;
using System.Collections.Generic;
namespace Playground
{
    /// <summary>
    /// ConditionGroup is an abstract construct of the composite pattern.
    /// <para>
    /// It's useful for grouped relational evaluations, such as logic gate operations.
    /// </para>
    /// </summary>
    [UnityEngine.HelpURL("https://justworksltd.gitlab.io/playground-docs/class_playground_1_1_condition_group.html")]
    public abstract class ConditionGroup : Condition, IEnumerable<Condition>
    {
        [System.NonSerialized] private List<Condition> _conditions = new List<Condition>();
        public Condition this[int i] => _conditions[i];
        public int ConditionsInGroup => _conditions.Count;
        public void AddCondition(Condition condition) => _conditions.Add(condition);
        public void RemoveCondition(Condition condition) => _conditions.Remove(condition);
        public void ClearConditionList()
        {
            foreach (Condition cond in _conditions)
            {
                Destroy(cond);
            }
            _conditions.Clear();
        }
        public IEnumerator<Condition> GetEnumerator() => _conditions.GetEnumerator();
        IEnumerator IEnumerable.GetEnumerator() => _conditions.GetEnumerator();

        /// <summary>
        /// used by the containers to insert id data into self
        /// </summary>
        /// <param name="cgc">teh container to update</param>
        /// <remarks>It does not return updated data to hold onto type of container</remarks>
        public T InsertMemberIdValues<T>(T cgc) where T : ConditionGroupContainer
        {
            cgc.conditionsIds = new uint[ConditionsInGroup];
            for (int i = 0; i < ConditionsInGroup; i++)
            {
                cgc.conditionsIds[i] = this[i].Id;
            }

            return cgc;
        }
    }
}