﻿using System.Runtime.Serialization;
using UnityEngine;
namespace Playground
{
    [System.Serializable]
    public class CuboidVolume : VolumeBase
    {
        public Vector3 min, max;
        public Matrix4x4 cuboidSpace;//NOTE: TODO? - Technically, we'd be good with just the rotation quaternion

        public CuboidVolume(Vector3 min, Vector3 max, Matrix4x4 cuboidSpace)
        {
            this.min = min;
            this.max = max;
            this.cuboidSpace = cuboidSpace;
        }

        public override bool InsideVolume(Vector3 worldPosition)
        {
            Vector3 p = cuboidSpace.MultiplyPoint3x4(worldPosition); //cuboid-local position

            return
                (min.x <= p.x && p.x <= max.x) &&
                (min.y <= p.y && p.y <= max.y) &&
                (min.z <= p.z && p.z <= max.z);
        }

       
    }

    [System.Serializable]
    public class SphereVolume : VolumeBase
    {
        public Vector3 center;
        public float radius;

        public SphereVolume(Vector3 center, float radius)
        {
            this.center = center;
            this.radius = radius;
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            throw new System.NotImplementedException();
        }

        public override bool InsideVolume(Vector3 worldPosition) => Vector3.Distance(worldPosition, center) <= radius;
    }

    [System.Serializable]
    public class CylinderVolume : VolumeBase
    {
        public Matrix4x4 cylinderSpace; //NOTE: TODO? - Technically, we'd be good with just the rotation quaternion
        public Vector3 center;
        public float radius;
        public float height;
        [SerializeField, HideInInspector]
        private Vector3 localUp;
        public CylinderVolume( Vector3 center, float height, float radius, Matrix4x4 cylinderSpace )
        {
            this.center = center;
            this.height = height;
            this.radius = radius;
            this.cylinderSpace = cylinderSpace;
            localUp = cylinderSpace.GetColumn(1); //discard vector.w
            localUp.Normalize(); //normalizing after discarding to not normalize by w
        }


        public override bool InsideVolume( Vector3 worldPosition )
        {
            Vector3 p = cylinderSpace.MultiplyPoint3x4(worldPosition); //cylinder-local position
            float distanceFromCenterOnLine = Vector3.Dot(localUp, p);
            if (Mathf.Abs(distanceFromCenterOnLine*2) > height) //the point was above or below the cylinder
                return false;
            Vector3 pointOnLine = localUp * distanceFromCenterOnLine;
            return Vector3.Distance(pointOnLine, p) <= radius;
        }
    }
}