﻿using UnityEngine;

namespace Playground
{
    // TODO: might be issues with use of abs in some cases?
    [System.Serializable]
    public class RotationCondition : Condition<Transform>
    {
        // TODO: maybe this should be Quaternion?
        [SerializeField] private Vector3 _targetEulerRotation;
        public Vector3 TargetEulerRoation
        {
            get { return _targetEulerRotation; }
            set { _targetEulerRotation = value; }
        }

        [SerializeField] private Vector3 _allowedEulerDeviation;
        public Vector3 AllowedEulerDeviation
        {
            get { return _allowedEulerDeviation; }
            set { _allowedEulerDeviation = value.Abs(); }
        }

        public override bool Evaluate()
        {
            bool IsInRange(Vector3 p) => p.x <= _allowedEulerDeviation.x &&
                                         p.y <= _allowedEulerDeviation.y &&
                                         p.z <= _allowedEulerDeviation.z;

            if (Relative)
            {
                return IsInRange(
                    (_targetEulerRotation - Relative.InverseTransformDirection(Context.transform.rotation.eulerAngles)).Abs()
                );
            }
            else
            {
                return IsInRange((_targetEulerRotation - Context.transform.rotation.eulerAngles).Abs());
            }
        }

        public override ConditionContainer GetDataContainer(ConditionValue conditionValue, bool inNestedGroupParam)
        {
            var container = base.GetDataContainer(conditionValue, inNestedGroupParam);
            var rcc = container.CopyBasicValues<RotationConditionContainer>();

            rcc.TargetEulerRoation = _targetEulerRotation;
            rcc.AllowedEulerDeviation = _allowedEulerDeviation;

            return rcc;
        }

    }
}

