﻿using UnityEngine;

namespace Playground
{
    /// <summary>
    /// Class to define limits in scene object's velocity 
    /// </summary>
    [System.Serializable]
    public sealed class VelocityCondition : Condition<Rigidbody>
    {
        #region context_components
        private Rigidbody _contexRigidbody;
        #endregion

        #region parameters
        /// <summary>
        /// Used too check upper limit for velocity. Mathf.Infinity for unlimited
        /// </summary>
        [SerializeField] private float _maxVelocity = Mathf.Infinity;
        public float MaxVelocity
        {
            get { return _maxVelocity; }
            set { _maxVelocity = value; }
        }

        /// <summary>
        /// Used too check minimum limit for velocity. Mathf.NegativeInfinity for unlimited
        /// </summary>
        [SerializeField] private float _minVelocity = Mathf.NegativeInfinity;
        public float MinVelocity
        {
            get { return _minVelocity; }
            set { _minVelocity = value; }
        }
        #endregion

        protected override void PreConfigure()
        {
            _contexRigidbody = Context.GetComponent<Rigidbody>();
        }


        public override bool Evaluate()
        {

            float contextVelocityMagnitude = _contexRigidbody.velocity.magnitude;

            if (Relative)
            {
                contextVelocityMagnitude = Mathf.Abs(contextVelocityMagnitude - Relative.velocity.magnitude);
            }

            return contextVelocityMagnitude <= _maxVelocity && contextVelocityMagnitude >= _minVelocity;
        }

        public override ConditionContainer GetDataContainer(ConditionValue conditionValue, bool inNestedGroupParam)
        {
            var container = base.GetDataContainer(conditionValue, inNestedGroupParam);
            var vcc = container.CopyBasicValues<VelocityConditionContainer>();
            vcc.MaxVelocity = _maxVelocity;
            vcc.MinVelocity = _minVelocity;

            return vcc;
        }
    }
}


