﻿using droid.Runtime.Utilities.Sensors;
using droid.Runtime.Utilities.Misc;
using UnityEngine;

namespace Playground
{
    [System.Serializable]
    public sealed class VolumeCondition : Condition<Transform>
    {

        [SerializeField] Collider _contextCollider;
        [SerializeField] Collider _volume;

        [SerializeField] bool volumeContainContext;

        protected override void PreConfigure()
        {
            _contextCollider = Context.GetComponent<Collider>();
            if(_contextCollider == null)
            {
                Debug.LogError("VolumeCondition failed to retrieve collider from context");
            }

            NeodroidUtilities.RegisterCollisionTriggerCallbacksOnChildren<ChildCollider3DSensor, Collider, Collision>(_volume, Context.transform, on_trigger_stay_child: IsVolumeContainingCollider);

        }

        // TODO: as of now we check if bounds contain other bounds, this should be colliders instead
        /// <summary>
        /// checks if volume contains the context collider 
        /// </summary>
        private void IsVolumeContainingCollider(GameObject collidingObject, Collider collider)
        {
            if (collidingObject != Context.gameObject)
                return;

            var volumeBounds = _volume.bounds;

            if (Relative)
                volumeBounds.center += Relative.transform.position;

            volumeContainContext = volumeBounds.Contains(_contextCollider.bounds.max) && volumeBounds.Contains(_contextCollider.bounds.min); 
        }

        /// <summary>
        /// Use this function to define the volume you are interested in
        /// </summary>
        /// <param name="collider">A trigger collider that represent the volume</param>
        public void SetVolume(Collider collider)
        {
            if (collider.isTrigger)
                _volume = collider;
            else
                Debug.LogWarning("cannot set volume to a non-trigger collider");
        }

        public override bool Evaluate()
        {
            return volumeContainContext;
        }
    }
}
