﻿using UnityEngine;

namespace Playground.Demo
{
    /// <summary>
    /// This was created to allow skipping jumping in vr to add conditions
    /// </summary>
    public class CreateConditions : MonoBehaviour
    {

        public GameObject relative;
        public SceneObject context;

        public Vector3 maxValue;
        public Vector3 minValue;

        public ConditionValue conditionValue;

        private ConditionGroup latestCondition;

        [ContextMenu("create position condition cuboid")]
        public void CreatePositionCondition()
        {
           
            var newCondition = context.gameObject.AddComponent<PositionCondition>();
            newCondition.SetRelative(relative.GetComponent<Transform>());
            newCondition.Initialize(context);

            var volume = new CuboidVolume(minValue, maxValue, transform.worldToLocalMatrix);
            newCondition._volume = volume;

            if (conditionValue == ConditionValue.goal)
                context.AddGoalCondition(newCondition);
            else if (conditionValue == ConditionValue.terminating)
                context.AddTerminatingCondition(newCondition);
        }

        [ContextMenu("create touch condition")]
        public void CreateTouchCondition()
        {

            var newCondition = context.gameObject.AddComponent<TouchCondition>();
            newCondition.SetRelative(relative.GetComponent<Collider>());
            newCondition.Initialize(context);

            if(conditionValue == ConditionValue.goal)
                context.AddGoalCondition(newCondition);
            else if (conditionValue == ConditionValue.terminating)
                context.AddTerminatingCondition(newCondition);
        }

        [ContextMenu("create velocity condition")]
        public void CreateVelocityCondition()
        {
            if (relative != null && relative.GetComponent<Rigidbody>() == null)
                return;

            var newCondition = context.gameObject.AddComponent<VelocityCondition>();

            if(relative != null)
                newCondition.SetRelative(relative.GetComponent<Rigidbody>());
            newCondition.Initialize(context);
            newCondition.MaxVelocity = maxValue.magnitude;
            newCondition.MinVelocity = minValue.magnitude;

            if (conditionValue == ConditionValue.goal)
                context.AddGoalCondition(newCondition);
            else if (conditionValue == ConditionValue.terminating)
                context.AddTerminatingCondition(newCondition);
        }

        [ContextMenu("create rotation condition")]
        public void CreateRotationCondition()
        {

            var newCondition = context.gameObject.AddComponent<RotationCondition>();

            newCondition.SetRelative(relative.GetComponent<Transform>());
            newCondition.Initialize(context);
            newCondition.AllowedEulerDeviation = maxValue;
            newCondition.TargetEulerRoation = minValue;

            if (conditionValue == ConditionValue.goal)
                context.AddGoalCondition(newCondition);
            else if (conditionValue == ConditionValue.terminating)
                context.AddTerminatingCondition(newCondition);
        }

        [ContextMenu("create or-group condition")]
        public void CreateOrGroupCondition()
        {
            var newCondition = context.gameObject.AddComponent<ConditionGroup_OR>();
            newCondition.Initialize(context);

            if(latestCondition == null)
            {
                if (conditionValue == ConditionValue.goal)
                    context.AddGoalCondition(newCondition);
                else if (conditionValue == ConditionValue.terminating)
                    context.AddTerminatingCondition(newCondition);
            }
            else
            {
                latestCondition.AddCondition(newCondition);
            }
            latestCondition = newCondition;
        }

        [ContextMenu("create and-group condition")]
        public void CreateAndGroupCondition()
        {
            var newCondition = context.gameObject.AddComponent<ConditionGroup_AND>();
            newCondition.Initialize(context);

            if (latestCondition == null)
            {
                if (conditionValue == ConditionValue.goal)
                    context.AddGoalCondition(newCondition);
                else if (conditionValue == ConditionValue.terminating)
                    context.AddTerminatingCondition(newCondition);
            }
            else
            {
                latestCondition.AddCondition(newCondition);
            }
            latestCondition = newCondition;
        }

        [ContextMenu("create position in group condition")]
        public void CreatePositionInOrGroupCondition()
        {

            var newCondition = context.gameObject.AddComponent<PositionCondition>();
            newCondition.SetRelative(relative.GetComponent<Transform>());
            newCondition.Initialize(context);

            var volume = new CuboidVolume(minValue, maxValue, transform.worldToLocalMatrix);
            newCondition._volume = volume;

            latestCondition.AddCondition(newCondition);
        }

        [ContextMenu("spam filter")]
        public void Clean()
        {
            var comp = gameObject.GetComponents<PositionCondition>();
            foreach (var item in comp)
            {
                Destroy(item);
            }
        }
    }
}

