﻿
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace Playground.Demo
{
    /// <summary>
    /// example evaluator that creates a random goal for an simple ball agent
    /// </summary>
    public class SceneEvaluatorCustomReset : Internal.SceneStateEvaluation
    {
        [SerializeField]
        public List<GameObject> wallObjects = new List<GameObject>();
        public GameObject targetObject;
        public GameObject sphereObject;

        private Vector3 sphereStart;

        #region RCPGenerationData
        public float objectDistance = 0.126f;
        public float curriculumRetraction = 0.001f;

        [Range(0,1)]
        public float targetAccuracy = 0.9f;
        [SerializeField]
        private float currentAccuracy;

        public int minimunEpisodeEachIteration = 100;
        private float lastEpisodeResult;

        [SerializeField]
        private int distanceEpisodeTotal;
        private int distanceEpisodeSuccess;
        #endregion

        protected override void PostSetup()
        {
            base.PostSetup();

            sphereStart = sphereObject.transform.position;

            distanceEpisodeTotal = 0;
            distanceEpisodeSuccess = 0;
        }

        public override void InternalReset()
        {
            base.InternalReset();

            // Reverse curriculum
            distanceEpisodeTotal += 1;
            if (lastEpisodeResult > 0.0f)
                distanceEpisodeSuccess += 1; 

            if (distanceEpisodeTotal != 0)
                currentAccuracy = (float)distanceEpisodeSuccess / distanceEpisodeTotal;

            // Place sceneobjects in new position
            StartCoroutine(PostInternalReset());
        }

        /// <summary>
        /// A hack for setting transforms in the scene with neo until we have a more solid solution
        /// </summary>
        /// <returns></returns>
        IEnumerator PostInternalReset()
        {
            // wait until neodroid move stuff
            yield return 0;

            Debug.Log(lastEpisodeResult);

            if (distanceEpisodeTotal > minimunEpisodeEachIteration)
            {
                if(currentAccuracy > targetAccuracy)
                {
                    distanceEpisodeTotal    = 0;
                    distanceEpisodeSuccess  = 0;

                    objectDistance -= curriculumRetraction;
                }
            }

            // do changes to scene after neodroid done its changes
            GenerateResetScene();
        }

        void GenerateResetScene()
        {
            var nexPosition = wallObjects[Random.Range(0, wallObjects.Count - 1)].transform.position;
            targetObject.transform.position = nexPosition;

            sphereObject.transform.position = Vector3.Lerp(sphereStart, nexPosition, objectDistance);
        }

        // hack to avoid 0 returns caused by coroutine
        public override float InternalEvaluate()
        {
            var currentEval = base.InternalEvaluate();
            if (currentEval > 0.00001f || currentEval < -0.00001f)
                currentEval = lastEpisodeResult;

            lastEpisodeResult = currentEval;
            return lastEpisodeResult;
        }

    }


}
