﻿using UnityEngine;

/// <summary>
/// Extensions for unity methods
/// </summary>
/// <!-- source: https://unity3d.com/learn/tutorials/topics/scripting/extension-methods -->
public static class ExtensionMethods
{
    /// <summary>
    /// Takes a vector3 and makes all members absolute values
    /// </summary>
    /// <param name="vec">The vector to turn to absolute</param>
    public static Vector3 Abs(this Vector3 vec) =>
        new Vector3(Mathf.Abs(vec.x), Mathf.Abs(vec.y), Mathf.Abs(vec.z));

    //https://answers.unity.com/questions/11363/converting-matrix4x4-to-quaternion-vector3.html
    /// <summary>
    /// Obtain the rotation as a quaterion of a TRS matrix
    /// </summary>
    /// <param name="m"></param>
    /// <returns>the rotation quaternion</returns>
    public static Quaternion GetRotation( this Matrix4x4 m ) => Quaternion.LookRotation(m.GetColumn(2), m.GetColumn(1));

    /// <summary>
    /// Obtain the position as a vector3 of a TRS matrix
    /// </summary>
    /// <param name="m"></param>
    /// <returns></returns>
    public static Vector3 GetPosition( this Matrix4x4 m ) => m.GetColumn(3);

    /// <summary>
    /// Obtain the scale as a vector3 of a TRS matrix
    /// <para>Warning: does not work with negative scales</para>
    /// </summary>
    /// <param name="m"></param>
    /// <returns></returns>
    public static Vector3 GetScale( this Matrix4x4 m )
    {
        Vector3 scale = new Vector3(
            m.GetColumn(0).magnitude,
            m.GetColumn(1).magnitude,
            m.GetColumn(2).magnitude
        );
        return scale;
    }

    public static void Shuffle<T>( this T[] arr )
    {
        int n = arr.Length;
        while (n > 1)
        {
            n--;
            int k = Random.Range(0, n + 1);
            T value = arr[k];
            arr[k] = arr[n];
            arr[n] = value;
        }
    }

}


