﻿using System;
using System.Collections.Generic;
using UnityEngine;

public interface ILabeledList { }
[Serializable]
public class LabeledList<TValue> : Dictionary<string, TValue>, ISerializationCallbackReceiver, ILabeledList
{

    [SerializeField]
    private List<string> _keys = new List<string>();
    [SerializeField]
    private List<TValue> _values = new List<TValue>();

    // save the dictionary to lists
    public void OnBeforeSerialize()
    {
        _keys.Clear();
        _values.Clear();
        foreach (var pair in this)
        {
            _keys.Add( pair.Key );
            _values.Add( pair.Value );
        }
    }
    // load dictionary from lists
    public void OnAfterDeserialize()
    {
        Debug.Assert(_keys.Count == _values.Count, $"Count of keys '{_keys.Count}' does not match count of values '{_values.Count}'. Dictionary compromised!");
        Clear();
        for (int i = 0; i < _keys.Count; i++)
            Add(_keys[i], _values[i]);
    }
}