﻿using UnityEngine;
using UnityEngine.EventSystems;

namespace Playground.Internal
{

    public interface IMouseClickCollision : IEventSystemHandler
    {
        void OnSetContext();
        void OnSetRelative(/*Condition*/);
    }

    public class SceneObjectSelectionManager : MonoBehaviour
    {
        [SerializeField] private SceneObject _contextObject;
        [SerializeField] private SceneObject _relativeObject;
        [SerializeField] private Condition _currentCondition;

        public SceneObject Context
        {
            get => _contextObject;
            set => _contextObject = value;
        }

        public SceneObject Relative
        {
            get => _relativeObject;
            set => _relativeObject = value;
        }
        public Condition CurrentCondition
        {
            get => _currentCondition;
            set => _currentCondition = value;
        }
        
        public void Update()
        {
            if (Input.anyKey)
            {
                if (Input.GetMouseButtonDown((int)PointerEventData.InputButton.Left))
                {
                    Debug.Log("LeftClick");
                    if (Physics.Raycast(Camera.main.ScreenPointToRay(Input.mousePosition), out var hit))
                    {
                        _contextObject = hit.collider.GetComponent<SceneObject>();
                        _contextObject.OnSetContext();
                        
                        showSelectBox = true;
                        position = CalculateRect();
                    }
                    else if(Input.GetKey(KeyCode.LeftShift))
                    {
                        _contextObject = null;
                        _relativeObject = null;
                        _currentCondition = null;

                        showSelectBox = false;
                    }
                }
                if (Input.GetMouseButtonDown((int)PointerEventData.InputButton.Right))
                {
                    Debug.Log("RightClick");
                    Debug.DrawLine(Camera.main.ScreenToWorldPoint(Input.mousePosition), Camera.main.ScreenToWorldPoint(Input.mousePosition) + Camera.main.transform.forward * 10, Color.red, 10);
                    if (Physics.Raycast(Camera.main.ScreenPointToRay(Input.mousePosition), out var hit))
                    {
                        _relativeObject = hit.collider.GetComponent<SceneObject>();
                        _relativeObject.OnSetRelative();

                        if (_currentCondition is IHasRelative conditionWithRelative)
                        {
                            var relativeComponent = _relativeObject.GetComponent(conditionWithRelative.RelativeType);
                            conditionWithRelative.SetRelative(relativeComponent);
                        }
                    }
                }
            }
        }


        private Rect CalculateRect()
        {
            return new Rect(
                Input.mousePosition.x, 
                Screen.height - Input.mousePosition.y, 
                208, 
                ConditionTypes.Types.Count * 28
            );
        }

        private Rect position;
        private bool showSelectBox;
        


        private void OnGUI()
        {
            if(showSelectBox)
            {
                GUI.Box(position, GUIContent.none);
                Rect item = new Rect() { x = position.x + 4, y = position.y + 4, height = 20, width = 200};
                foreach (var type in ConditionTypes.Types)
                {
                    if(GUI.Button(item, type.Name))
                    {
                        _currentCondition = (Condition)_contextObject.gameObject.AddComponent(type);
                        _currentCondition.Initialize(_contextObject);

                        _contextObject.AddGoalCondition(_currentCondition);
                        if(_currentCondition is IHasRelative condWithRelative)
                        {
                            var relativeComponent = _relativeObject.GetComponent(condWithRelative.RelativeType);
                            condWithRelative.SetRelative(relativeComponent);
                        }
                        showSelectBox = false;
                        ConditionTypes.CreateConditionEditorObject(_currentCondition);
                    }
                    item.y += 28;
                }
            }
        }
    }
}