﻿using System.Runtime.Serialization;
using UnityEngine;

namespace Playground.SaveSystem
{
    /// <summary>
    /// Used to serialize unity transforms
    /// </summary>
    public sealed class TransformSurrogate : ISerializationSurrogate
    {
        public void GetObjectData(object obj, SerializationInfo info, StreamingContext context)
        {
            var transform = (Transform)obj;

            for (int i = 0; i < 2; i++)
            {
                // serialize position and scale
                var represent = ((i < 1) ? "position" : "scale");
                var data = ((i < 1) ? transform.position : transform.localScale);
                for (int j = 0; j < 3; j++)
                {
                    info.AddValue(string.Format("{0}.{1}", represent, j), data[j]);
                }
            }

            for (int i = 0; i < 4; i++)
            {
                info.AddValue(string.Format("quaternion.{0}", i), transform.rotation[i]);
            }
        }


        public object SetObjectData(object obj, SerializationInfo info, StreamingContext context, ISurrogateSelector selector)
        {

            var transform = (Transform)obj;

            // deserialize position and scale
            for (int i = 0; i < 2; i++)
            {
                var represent = ((i < 1) ? "position" : "scale");
                var data = ((i < 1) ? transform.position : transform.localScale);
                for (int j = 0; j < 3; j++)
                {
                    data[j] = (float)info.GetValue(string.Format("{0},{1}", represent, j), typeof(float));
                }

                if (i < 1)
                    transform.position = data;
                else
                    transform.localScale = data;
            }

            // deserialize rotaion
            var quatData = transform.rotation;
            for (int i = 0; i < 4; i++)
            {
                quatData[i] = (float)info.GetValue(string.Format("quaternion.{0}", i), typeof(float));
            }
            transform.rotation = quatData;

            return transform;
        }

    }

}