﻿using UnityEngine;
using UnityEngine.EventSystems;
namespace Playground
{
    [HelpURL("https://justworksltd.gitlab.io/playground-docs/class_playground_1_1_scene_object.html")]
    [RequireComponent(typeof(droid.Runtime.Prototyping.Observers.Transform.EulerTransformObserver))] //TODO: Each condition should probably hold this themselves, in order to allow each condition to define what needs to be observed in order to "understand" why a condition is/-not satisfied
    public class SceneObject : MonoBehaviour, Internal.IMouseClickCollision, IPointerClickHandler, IPointerEnterHandler, IPointerExitHandler
    {
#pragma warning disable 0649
        [SerializeField] private Material _contextObjectMaterial;
        [SerializeField] private Material _relativeObjectMaterial;
#pragma warning restore 0649

#if UNITY_EDITOR
        //TODO: Use EditorPrefs instead :think:
        [SerializeField] private bool _editor_showConditions;

        private void Reset()
        {
            ValidateTerminatingList();
            ValidateGoalList();
        }
#endif

        public uint Id;

        [SerializeField] private ConditionGroup_OR _terminatingConditions;
        public ConditionGroup_OR TerminatingConditions
        {
            get { return _terminatingConditions; }
            set { _terminatingConditions = value; } 
        }
        [SerializeField] private ConditionGroup_OR _goalConditions;
        public ConditionGroup_OR GoalConditions
        {
            get { return _goalConditions; }
            set { _goalConditions = value; }
        }

        // Possibly put context and relative materials here
        Material _originalMaterial;
        Renderer _renderer;


        private void ValidateTerminatingList()
        {
            if (!_terminatingConditions)
            {
                _terminatingConditions = gameObject.AddComponent<ConditionGroup_OR>();
                _terminatingConditions.Initialize(this);
            }
        }

        private void ValidateGoalList()
        {
            if (!_goalConditions)
            {
                _goalConditions = gameObject.AddComponent<ConditionGroup_OR>();
                _goalConditions.Initialize(this);
            }
        }

        private void Awake()
        {
            ValidateTerminatingList();
            ValidateGoalList();

            Id = SaveSystem.ConditionDataManager.Instance.GetUniqueId();
        }

        private void Start()
        {
            _renderer = GetComponent<Renderer>();
            _originalMaterial = _renderer.material;
        }

        //TODO: Wrap code related to setting context/relative into these instead of spread out in the other calls
        public void OnSetContext()
        {
#if PLAYGROUND_DEBUG
            Debug.Log($"SCENE OBJECT: {name} was set as context!");
#endif
        }
        public void OnSetRelative()
        {
#if PLAYGROUND_DEBUG
            Debug.Log($"SCENE OBJECT: {name} was set as a relative!");
#endif
        }//TODO

        public void SetMaterial(Material mat)
        {
            if(_renderer != null)
                _renderer.material = mat;
        }

        public void RevertMaterial()
        {
            if(_renderer != null)
                _renderer.material = _originalMaterial;
        }

        public void AddGoalCondition(Condition condition)
        {
            ValidateGoalList();
            _goalConditions.AddCondition(condition);
        }

        public void AddTerminatingCondition(Condition condition)
        {
            ValidateTerminatingList();
            _terminatingConditions.AddCondition(condition);
        }

        public void RemoveAllConditions()
        {
            GoalConditions.ClearConditionList();
            TerminatingConditions.ClearConditionList();
        }

        public void OnPointerClick( PointerEventData eventData )
        {
            GameManager.Instance.laserPointer.Select(this);
        }
        public void OnPointerEnter( PointerEventData eventData )
        {
            var laserPointer = GameManager.Instance.laserPointer;
            if (laserPointer.Mode == UI.LaserPointer.SelectionMode.Context)
                SetMaterial(laserPointer.ContextSelectionMaterial);
            else
                SetMaterial(laserPointer.RelativeSelectionMaterial);
        }

        public void OnPointerExit( PointerEventData eventData )
        {
            RevertMaterial();
        }

    }
}
