﻿using droid.Runtime.Prototyping.Evaluation;
using System.Collections.Generic;

namespace Playground.Internal
{
    public class SceneStateEvaluation : ObjectiveFunction
    {
        protected const int DEFAULT_NR_OF_SAMPLES = 1000;

        public List<SceneObject> sceneObjects = new List<SceneObject>();

        protected override void PostSetup()
        {
            sceneObjects.Clear();
            foreach (SceneObject obj in FindObjectsOfType<SceneObject>())
            {
                sceneObjects.Add(obj);
            }
        }

        public override float InternalEvaluate()
        {
            foreach (var sceneObject in sceneObjects)
            {
                if (sceneObject.TerminatingConditions.ConditionsInGroup > 0 && sceneObject.TerminatingConditions.Evaluate())
                {
                    ParentEnvironment?.Terminate($"Entered terminating state on object: { sceneObject }");
                    return -1; //NOTE: this might not even matter seing as the environment is terminated
                }
            }

            // Checking goal conditions after termination conditions
            foreach (var sceneObject in sceneObjects)
            {
                if (sceneObject.GoalConditions.ConditionsInGroup > 0 && sceneObject.GoalConditions.Evaluate())
                {
                    ParentEnvironment?.Terminate($"Entered goal state on object: { sceneObject }");
                    return 1;
                }
            }
            return 0; //NOTE: no conditions == success???? idk
        }
        public override void InternalReset()
        {
            //GO TO NEXT SAMPLE?

            //TODO capture state of scene on startup in order to reset back to it
            //throw new System.NotImplementedException();
        }

        public void GenerateState( float percent, uint samples, params SceneObject[] objects )
        {
            // construct a randomized list of the given objects; dubbed "the frontier stack"
            // get number of sceneobjects with conditions
            // get total number of conditions across the entire scene
            // get nr of sceneobject's conditions to satisfy. -> and number of sceneobjects whole state to satisfy.
            // loop through the sceneobjects and conditions in question, try to generate samples that are valid states.
            //     NOTE: conditions can implement their own "Shake" function in order to get the correct percentage for a given sceneobject
            // TODO: MORE

            objects.Shuffle(); //use the same shuffled configuration for all the preceding samples.

            //int nrOfSceneObjects = objects.Length;
            //int nrOfConditions   = 0;
            ////get the shallow count of conditions
            //foreach (var obj in objects)
            //{
            //    nrOfConditions += (obj.GoalConditions.ConditionsInGroup > 0) ? 1:0;
            //    nrOfConditions += (obj.TerminatingConditions.ConditionsInGroup > 0) ? 1 : 0;
            //}
        }

        public void GenerateState( float percent, uint samples = DEFAULT_NR_OF_SAMPLES)
        {
            GenerateState(percent, samples, sceneObjects.ToArray());
        }
    }
}
