﻿using System;
using UnityEngine;

namespace Playground.UI
{
    public class ConditionButton : MenuButton
    {
        enum ConditionType
        {
            PositionCondition,
            RotationCondition,
            TouchCondition,
            VelocityCondition
        }
#pragma warning disable 0649
        [SerializeField] private ConditionType _label;
        [SerializeField] private bool _isEnabled;
#pragma warning restore 0649

        public bool IsEnabled
        {
            get { return _isEnabled; }
            set { _isEnabled = value; }
        }

        public override void OnClick()
        {
            if(Time.timeSinceLevelLoad - _lastTimeClicked > _clickCooldown)
            {
                base.OnClick(); // This also check for last time clicked, redundant TODO
                _isEnabled = !_isEnabled;
                if (_isEnabled)
                {
                    image.color = button.colors.selectedColor;
                    if (_label == ConditionType.PositionCondition)
                        AddCondition(typeof(PositionCondition));

                    else if(_label == ConditionType.RotationCondition)
                        AddCondition(typeof(RotationCondition));

                    else if (_label == ConditionType.TouchCondition)
                        AddCondition(typeof(TouchCondition));

                    else if (_label == ConditionType.VelocityCondition)
                        AddCondition(typeof(VelocityCondition));
                }
                else
                {
                    image.color = button.colors.normalColor;
                    if (_label == ConditionType.PositionCondition)
                        RemoveCondition(typeof(PositionCondition));

                    else if (_label == ConditionType.RotationCondition)
                        RemoveCondition(typeof(RotationCondition));

                    else if (_label == ConditionType.TouchCondition)
                        RemoveCondition(typeof(TouchCondition));

                    else if (_label == ConditionType.VelocityCondition)
                        RemoveCondition(typeof(VelocityCondition));
                }
            }
        }

        public override void OnHover()
        {
            base.OnHover();
            if (_isEnabled)
            {
                image.color = button.colors.selectedColor;
            }
        }

        public override void DeHover()
        {
            base.DeHover();
            if (_isEnabled)
            {
                image.color = button.colors.selectedColor;
            }
        }

        protected override void OnEnable()
        {
            base.OnEnable();
            _lastTimeClicked = Time.timeSinceLevelLoad;
            image.color = button.colors.normalColor;
            _isEnabled = false;
        }

        private void AddCondition(Type t)
        {
            FindObjectOfType<ConditionMenuController>().AddConditionToList(t);
        }

        private void RemoveCondition(Type t)
        {
            FindObjectOfType<ConditionMenuController>().RemoveConditionFromList(t);
        }
    }
}
