﻿using UnityEngine;

namespace Playground.UI
{

    [RequireComponent(typeof(SceneObject))]
    public class ConditionIndicator : MonoBehaviour
    {
        [HideInInspector]
        public bool visible = true;
        
        [SerializeField]
        private bool _debug= false;
        [SerializeField]
        private bool _debugSpriteRendering = false;
        [SerializeField]
        private float _spriteSize = 0.08f;
        [SerializeField]
        private float _spriteOffset = 0.2f;

        private Sprite _completeSpirte;
        private Sprite _fatalSpirte;
        private Sprite _neutralSpirte;
        private Sprite _debugSprite;


        private Transform       _player;        // The steamVR player object.
        private Transform       _child;         // Intermediate object to fix scaling.
        private Transform       _grandChild;    // The actual indicator the player sees.
        private SpriteRenderer  _sprite;        // The spriteRender on that child object.
        private SceneObject     _sceneObject;
        
        private void Awake()
        {
            _completeSpirte =   Resources.Load<Sprite>("Complete");   // Note: The sprites need to be located in Assets/Resources for this Load<>() to work.
            _fatalSpirte =      Resources.Load<Sprite>("Fatal");
            _neutralSpirte =    Resources.Load<Sprite>("Neutral");
            _debugSprite =      Resources.Load<Sprite>("Debug");
        }

        // Start is called before the first frame update
        private void Start()
        {
            _player =  (GameManager.Instance.usingVR) ? 
                GameManager.Instance.player.GetComponentInChildren<Camera>().transform : 
                GameManager.Instance.player;


            _child = new GameObject("ConditionIndicator").transform;
            _child.parent = transform;
            _child.localRotation = Quaternion.identity; 
            Vector3 s = transform.lossyScale;
            _child.localScale = new Vector3(1/s.x, 1/s.y, 1/s.z);

            _grandChild = new GameObject("Sprite", typeof(SpriteRenderer)).transform;
            _grandChild.parent = _child;
            _grandChild.localScale = Vector3.one * _spriteSize;

            _sprite = _grandChild.GetComponent<SpriteRenderer>();
            _sprite.sprite = _neutralSpirte;

            _sceneObject = GetComponent<SceneObject>();
        }

        // Update is called once per frame
        private void Update()
        {
            Vector3 toPlayer = (transform.position - _player.position).normalized;
            toPlayer.y = 0;

            _grandChild.position = transform.position + Vector3.up * _spriteOffset; // TODO: Move the sprite dynamicly to always be just above the Scene Object.
            _grandChild.transform.rotation = Quaternion.LookRotation(toPlayer, Vector3.up);


            if (_sceneObject.TerminatingConditions.ConditionsInGroup > 0 && _sceneObject.TerminatingConditions.Evaluate())
            {
                _sprite.sprite = _fatalSpirte;
                if (_debug) Debug.Log("Fatal state on SceneObject: " + gameObject.name);
            }
            else if (_sceneObject.GoalConditions.ConditionsInGroup > 0 && _sceneObject.GoalConditions.Evaluate())
            {
                _sprite.sprite = _completeSpirte;
                if (_debug) Debug.Log("Goal state on SceneObject: " + gameObject.name);
            }
            else
            {
                _sprite.sprite = _neutralSpirte;
            }
            
            if (!visible || (_sceneObject.GoalConditions.ConditionsInGroup == 0 && _sceneObject.TerminatingConditions.ConditionsInGroup == 0))
            {
                _sprite.enabled = false;
            }
            else
            {
                _sprite.enabled = true;
            }

            if (_debugSpriteRendering)
            {
                _sprite.enabled = true;
                _sprite.sprite = _debugSprite;
            }

            if (_debug) Debug.Log(  "SceneObject condtions. Goal: " + _sceneObject.GoalConditions.ConditionsInGroup
                                    +" \t Fatal: " + _sceneObject.TerminatingConditions.ConditionsInGroup);

        }
    }

}
