﻿using UnityEngine;
using UnityEngine.UI;

namespace Playground.UI
{
    /// <summary>
    /// The condition menu shown when interfacing with adding conditions
    /// </summary>
    public class ConditionMenu : MonoBehaviour
    {
        public enum MenuStatus
        {
            ExitMenu            = -1,
            VerifyObject        = 0,
            ChooseTRelative     = 1,
            VerifyTRelative     = 2,
            ChooseConditions    = 3,
            ApplyConditions     = 4,
            ChooseStateType     = 5, // Terminating state or goal state
            Completed           = 6
        }
#pragma warning disable 0649
        private Vector3 _flipVector = new Vector3(0, 180, 0);
        private Transform _demonstratorHead;
        [SerializeField] private GameObject _choiceConfirmationMenu;
        [SerializeField] private GameObject _chooseConditionsMenu;
        [SerializeField] private GameObject _chooseStateTypeMenu;
        [SerializeField] private GameObject _applyConditionsMenu;
        [SerializeField] private Text _statusText;
#pragma warning restore 0649
        public Text StatusText
        {
            get { return _statusText; }
            set { _statusText = value; }
        }

        private void Start()
        {
            _demonstratorHead = GameObject.Find("FollowHead").transform;
        }

        private void Update()
        {
            transform.LookAt(_demonstratorHead);
            transform.Rotate(_flipVector);
        }

        public void UpdateStatusText(string text)
        {
            _statusText.text = text;
        }

        public void UpdateMenu(MenuStatus status)
        {
            if (status == MenuStatus.ChooseTRelative)
            {
                DisableAllMenus();
                UpdateStatusText("Choose what object the conditions should be relative to");
            }
            else if(status == MenuStatus.ChooseConditions)
            {
                EnableChooseConditionsMenu();
                UpdateStatusText("Choose what conditions to enable for this object");
            }
            else if(status == MenuStatus.ApplyConditions)
            {
                // TODO: This is not implemented yet, what should be done is:
                // Go through each type of condition that is added, and interact to apply them to the object
                // For now this is for DEBUG:
                EnableApplyConditionsMenu(); // DEBUG, probably don't need this anymore
                UpdateStatusText("Press continue to proceed when done applying condition");
            }
            else if(status == MenuStatus.ChooseStateType)
            {
                // This menu's onClick method is not implemented yet either, it's supposed to tell if the current condition
                // is supposed to trigger a goal state or terminating state
                EnableChooseStateTypeMenu();
                UpdateStatusText("What does these conditions trigger?"); // Goal state, or terminating state?
            }
            else if(status == MenuStatus.Completed)
            {
                DisableAllMenus();
                UpdateStatusText("You have completed the setup of these conditions on this object");
            }
            else
            {
                EnableChoiceConfirmationMenu();
            }
        }

        private void DisableAllMenus()
        {
            if (_choiceConfirmationMenu.activeSelf)
                _choiceConfirmationMenu.SetActive(false);

            if (_chooseConditionsMenu.activeSelf)
                _chooseConditionsMenu.SetActive(false);

            if (_chooseStateTypeMenu.activeSelf)
                _chooseStateTypeMenu.SetActive(false);

            if (_applyConditionsMenu.activeSelf)
                _applyConditionsMenu.SetActive(false);
        }

        private void EnableChoiceConfirmationMenu()
        {
            if (!_choiceConfirmationMenu.activeSelf)
                _choiceConfirmationMenu.SetActive(true);

            if (_chooseConditionsMenu.activeSelf)
                _chooseConditionsMenu.SetActive(false);

            if (_chooseStateTypeMenu.activeSelf)
                _chooseStateTypeMenu.SetActive(false);

            if (_applyConditionsMenu.activeSelf)
                _applyConditionsMenu.SetActive(false);
        }

        private void EnableChooseConditionsMenu()
        {
            if (_choiceConfirmationMenu.activeSelf)
                _choiceConfirmationMenu.SetActive(false);

            if (!_chooseConditionsMenu.activeSelf)
                _chooseConditionsMenu.SetActive(true);

            if (_chooseStateTypeMenu.activeSelf)
                _chooseStateTypeMenu.SetActive(false);

            if (_applyConditionsMenu.activeSelf)
                _applyConditionsMenu.SetActive(false);
        }

        private void EnableChooseStateTypeMenu()
        {
            if (_choiceConfirmationMenu.activeSelf)
                _choiceConfirmationMenu.SetActive(false);

            if (_chooseConditionsMenu.activeSelf)
                _chooseConditionsMenu.SetActive(false);

            if (!_chooseStateTypeMenu.activeSelf)
                _chooseStateTypeMenu.SetActive(true);

            if (_applyConditionsMenu.activeSelf)
                _applyConditionsMenu.SetActive(false);
        }

        private void EnableApplyConditionsMenu()
        {
            if (_choiceConfirmationMenu.activeSelf)
                _choiceConfirmationMenu.SetActive(false);

            if (_chooseConditionsMenu.activeSelf)
                _chooseConditionsMenu.SetActive(false);

            if (_chooseStateTypeMenu.activeSelf)
                _chooseStateTypeMenu.SetActive(false);

            if (!_applyConditionsMenu.activeSelf)
                _applyConditionsMenu.SetActive(true);
        }

    }
}
