﻿using Playground.Internal;
using System;
using System.Collections.Generic;
using UnityEngine;
using Valve.VR;

// TODO: Remove Debug logs
// TODO: Add a close menu button
namespace Playground.UI
{
    /// <summary>
    /// Class for user interactions on the ConditionMenu
    /// </summary>
    [Obsolete("Use TabletMenu instead")]
    public class ConditionMenuController : MonoBehaviour
    {
        [SerializeField]
        private bool _debug = false;
        private Transform _demonstratorHead;
        private MenuButton _previousMenuButton;
        private MenuButton _currentMenuButton;
        private Valve.VR.InteractionSystem.Hand _leftHand;
        private Valve.VR.InteractionSystem.Hand _rightHand;
        private SteamVR_Action_Boolean _interactUIButton = SteamVR_Input.GetAction<SteamVR_Action_Boolean>("default", "interactui");

        private float _menuCooldown;
#pragma warning disable 0649
        [SerializeField] private int _currentConditionIndex;
        [SerializeField] private float _menuToggleTimer = 0.5f;
        [SerializeField] private float _controllerTargetRadius = 0.2f;
        [SerializeField] private LayerMask _interactableLayerMask;
        [SerializeField] private LayerMask _userInterfaceLayerMask;
        [SerializeField] private LineRenderer _controllerRay;
        [SerializeField] private ConditionMenu _conditionMenu;
        [SerializeField] private List<Type> _conditionsToAdd; 
        [SerializeField] private List<Condition> _conditions;
        [SerializeField] private List<IConditionEditor> _editors;
        [SerializeField] private Material _contextObjectMaterial;
        [SerializeField] private Material _relativeObjectMaterial;
#pragma warning restore 0649

        /// <summary>
        /// Gives feedback whether the condition menu currently is active or not
        /// </summary>
        [SerializeField] private bool _menuActive;
        public bool IsMenuActive
        {
            get { return _menuActive; }
            set { _menuActive = value; }
        }

        [SerializeField] private SceneObject _currentTargetedObject;
        public SceneObject CurrentTargetedObject
        {
            get { return _currentTargetedObject; }
            set { _currentTargetedObject = value; }
        }

        [SerializeField] private SceneObject _currentRelative;
        public SceneObject CurrentRelative
        {
            get { return _currentRelative; }
            set { _currentRelative = value; }
        }

        [SerializeField] private ConditionMenu.MenuStatus _menuStatus;
        public ConditionMenu.MenuStatus MenuStatus
        {
            get { return _menuStatus; }
            set { _menuStatus = value; }
        }

        private void Start()
        {
            _menuCooldown       = 0;
            _menuActive         = false;
            _leftHand           = GameObject.Find("LeftHand").GetComponent<Valve.VR.InteractionSystem.Hand>();
            _rightHand          = GameObject.Find("RightHand").GetComponent<Valve.VR.InteractionSystem.Hand>();
            _demonstratorHead   = GameObject.Find("FollowHead").transform;
            if (_conditionMenu == null)
                _conditionMenu  = GameObject.Find("ConditionMenu").GetComponent<ConditionMenu>();
            _conditions         = new List<Condition>();
            _conditionsToAdd    = new List<Type>();
            _editors            = new List<IConditionEditor>();
        }

        private void Update()
        {
            if (!_menuActive)
            {
                HandleMenuButtonInput();
            }
            else if (_menuActive)
            {
                if(_menuStatus != ConditionMenu.MenuStatus.ChooseTRelative && _menuStatus != ConditionMenu.MenuStatus.ApplyConditions)
                {
                    _currentMenuButton = RaycastFromHand();
                    if (_previousMenuButton != null && _previousMenuButton != _currentMenuButton)
                        _previousMenuButton.DeHover();
                    if (_currentMenuButton == null)
                        return;

                    _previousMenuButton = _currentMenuButton;
                    HandleButtonInteraction();
                }
                else if(_menuStatus == ConditionMenu.MenuStatus.ChooseTRelative)
                {
                    DisableRayCastFromHand(); // Redundant calling, should be changed
                    GetRelativeFromUserInput();
                    _currentConditionIndex = 0;
                }
                else if(_menuStatus == ConditionMenu.MenuStatus.ApplyConditions)
                {
                    DisableRayCastFromHand(); // Redundant calling, should be changed
                    //CheckForProceedInput(); Causes button input conflict with toggle hand tablet menu. 
                    if (_currentConditionIndex >= _conditionsToAdd.Count)
                    {
                        if (_debug) Debug.Log("WENT TO PROCEED IN MENU");
                        MenuProceed();
                        foreach (IConditionEditor editor in _editors)
                        {
                            editor.DestroyEditor();
                        }
                    }
                    else if(_currentConditionIndex > _conditions.Count - 1 && _conditionsToAdd.Count > _conditions.Count) // Kind of an hack, but this is to make sure only build the UI once 
                    {
                        if (_debug) Debug.Log("Trying to ADD CONDITION");
                        Condition newCondition = (Condition)_currentTargetedObject.gameObject.AddComponent(_conditionsToAdd[_currentConditionIndex]);
                        _conditions.Add(newCondition);
                        _conditions[_currentConditionIndex].Initialize(_currentTargetedObject.GetComponent<SceneObject>());
                        if(_conditions[_currentConditionIndex] is IHasRelative condition)
                        {
                            condition.SetRelative(_currentRelative.transform);
                        }
                        _editors.Add(ConditionTypes.CreateConditionEditorObject(_conditions[_currentConditionIndex]));
                    }
                    if (_debug) Debug.Log("Conditions in list to add:" + _conditionsToAdd.Count);
                }
            }
        }

        /// <summary>
        /// Handles user input of enabling or disabling the condition menu
        /// </summary>
        private void HandleMenuButtonInput()
        {
            _menuCooldown -= Time.deltaTime;
            if (_menuCooldown > 0)
                return;
            else if (_menuCooldown < 0)
                _menuCooldown = 0;

            // TODO: Should possibly be a callback on state changed instead of actively checking current state of the button
            if (_interactUIButton.GetState(SteamVR_Input_Sources.LeftHand))
            {
                TryGetTargetedObject(_leftHand.transform.position);
            }
            else if(_interactUIButton.GetState(SteamVR_Input_Sources.RightHand))
            {
                TryGetTargetedObject(_rightHand.transform.position);
            }
        }

        private void TryGetTargetedObject(Vector3 position)
        {
            _currentTargetedObject = GetClosestInteractable(position);
            if (_currentTargetedObject == null)
                return;
            _menuCooldown = _menuToggleTimer;
            _menuActive = true;
            InstantiateMenuOnTarget();
            _menuStatus = ConditionMenu.MenuStatus.VerifyObject;
            _conditionMenu.UpdateMenu(_menuStatus);
            // This should probably be done in another way, but works for now
            _conditionMenu.UpdateStatusText("Is this the object you wanted to put conditions on?" + Environment.NewLine + _currentTargetedObject.name);
            _currentTargetedObject.SetMaterial(_contextObjectMaterial);
            _currentConditionIndex = 0;
        }

        /// <summary>
        /// Finds closest interactable and returns it, mainly used when targeting an object to put conditions on
        /// </summary>
        /// <param name="position">The position to query from, usually the hand/controllers position</param>
        /// <returns></returns>
        // Should this be returning SceneObject?
        // TODO: This should be returning the parent object with Rigidbody and not the object with the collider as it could just be a child object
        private SceneObject GetClosestInteractable(Vector3 position)
        {
            Collider[] collisions = Physics.OverlapSphere(position, _controllerTargetRadius, _interactableLayerMask);

            float closestDistance = Mathf.Infinity;
            Collider closestObject = null;
            foreach(Collider coll in collisions)
            {
                float distance = (position - coll.transform.position).sqrMagnitude;
                if(distance < closestDistance)
                {
                    closestDistance = distance;
                    closestObject = coll;
                }
            }
            if (closestObject != null)
                return closestObject.gameObject.GetComponent<SceneObject>();
            else
                return null;
        }

        private void InstantiateMenuOnTarget()
        {
            _conditionMenu.gameObject.SetActive(true);
            var menuPosition = new Vector3(_currentTargetedObject.transform.position.x,
                                           _currentTargetedObject.transform.position.y + 0.5f,
                                           _currentTargetedObject.transform.position.z);
            // Doing this check as it sometimes gave null ref
            if (_demonstratorHead == null)
                _demonstratorHead = GameObject.Find("FollowHead").transform;
            Vector3 offset = _demonstratorHead.forward;
            _conditionMenu.transform.position = new Vector3(menuPosition.x + (offset.x / 3), menuPosition.y, menuPosition.z + (offset.z / 3));
        }

        // Not sure if this will be a too heavy way of doing this, but hopefully will work ok
        private MenuButton RaycastFromHand()
        {
            if (!_controllerRay.gameObject.activeSelf)
                _controllerRay.gameObject.SetActive(true);

            MenuButton returnButton = null;
            float rayLength = 5;
            var ray = new Ray(_rightHand.transform.position, _rightHand.transform.forward);
            Vector3 endPosition = _rightHand.transform.position + (rayLength * _rightHand.transform.forward);

            if(Physics.Raycast(ray, out RaycastHit raycastHit, rayLength, _userInterfaceLayerMask.value))
            {
                endPosition = raycastHit.point;
                returnButton = raycastHit.transform.GetComponent<MenuButton>();
            }
            _controllerRay.SetPosition(0, _rightHand.transform.position);
            _controllerRay.SetPosition(1, endPosition);

            return returnButton;
        }

        private void DisableRayCastFromHand()
        {
            if (_controllerRay.gameObject.activeSelf)
                _controllerRay.gameObject.SetActive(false);
        }

        public void ExitMenu()
        {
            _menuActive = false;
            _conditionMenu.gameObject.SetActive(false);
            DisableRayCastFromHand();
            _currentTargetedObject?.RevertMaterial();
            _currentRelative?.RevertMaterial();
            _conditions.Clear();
            _conditionsToAdd.Clear();
            _editors.Clear();
            _currentConditionIndex = 0;
        }

        public void MenuProceed()
        {
            _menuStatus += 1;
            _conditionMenu.UpdateMenu(_menuStatus);
        }

        public void MenuGoBack()
        {
            _menuStatus -= 1;
            if (_menuStatus == ConditionMenu.MenuStatus.ExitMenu)
                ExitMenu();
            else
                _conditionMenu.UpdateMenu(_menuStatus);
        }

        private void HandleButtonInteraction()
        {
            _currentMenuButton.OnHover();
            if (_interactUIButton.GetState(SteamVR_Input_Sources.RightHand))
            {
                _currentMenuButton.OnClick();
            }
        }

        // Some duplicated code from HandleMenuButtonInput() here but it works slightly different, just having it like this for now
        private void GetRelativeFromUserInput()
        {
            _menuCooldown -= Time.deltaTime;
            if (_menuCooldown > 0)
                return;
            else if (_menuCooldown < 0)
                _menuCooldown = 0;

            // TODO: Should possibly be a callback on state changed instead of actively checking current state of the button
            // TODO: Known bug: Lets you put TRelative on the same object as targeted object
            if (_interactUIButton.GetState(SteamVR_Input_Sources.LeftHand))
            {
                _currentRelative = GetClosestInteractable(_leftHand.transform.position);
                if (_currentRelative == null || _currentRelative == _currentTargetedObject)
                    return;
                _menuCooldown = _menuToggleTimer;
                _menuStatus = ConditionMenu.MenuStatus.VerifyTRelative;
                _conditionMenu.UpdateMenu(_menuStatus);
                // This should probably be done in another way, but works for now
                _conditionMenu.UpdateStatusText("Is this the object you want to have the conditions relative to?" + Environment.NewLine + _currentRelative.name);
                _currentRelative.SetMaterial(_relativeObjectMaterial);
            }
            else if (_interactUIButton.GetState(SteamVR_Input_Sources.RightHand))
            {
                _currentRelative = GetClosestInteractable(_rightHand.transform.position);
                if (_currentRelative == null || _currentRelative == _currentTargetedObject)
                    return;
                _menuCooldown = _menuToggleTimer;
                _menuStatus = ConditionMenu.MenuStatus.VerifyTRelative;
                _conditionMenu.UpdateMenu(_menuStatus);
                // This should probably be done in another way, but works for now
                _conditionMenu.UpdateStatusText("Is this the object you want to have the conditions relative to?" + Environment.NewLine + _currentRelative.name);
                _currentRelative.SetMaterial(_relativeObjectMaterial);
            }
        }

        private void CheckForProceedInput()
        {
            _menuCooldown -= Time.deltaTime;
            if (_menuCooldown > 0)
                return;
            else if (_menuCooldown < 0)
                _menuCooldown = 0;

            if (_interactUIButton.GetState(SteamVR_Input_Sources.LeftHand) || _interactUIButton.GetState(SteamVR_Input_Sources.RightHand))
            {
                _menuCooldown = _menuToggleTimer;
                GoToNextConditionInList();
            }
        }

        public void AddConditionToList(Type t)
        {
            if (!_conditionsToAdd.Contains(t))
                _conditionsToAdd.Add(t);
        }

        public void RemoveConditionFromList(Type t)
        {
            if (_conditionsToAdd.Contains(t))
                _conditionsToAdd.Remove(t);
        }

        public void GoToNextConditionInList()
        {
            _currentConditionIndex += 1;
        }

        public void AddConditionsAsGoal()
        {
            foreach(Condition c in _conditions)
            {
                _currentTargetedObject.AddGoalCondition(c);
            }
        }

        public void AddConditionsAsTerminating()
        {
            foreach (Condition c in _conditions)
            {
                _currentTargetedObject.AddTerminatingCondition(c);
            }
        }

    }
}
