﻿namespace Playground.UI
{
    [ConditionEditor(typeof(PositionCondition))]
    public class PositionConditionEditor : ConditionEditor<PositionCondition>
    {
        ValueHandle<int> _dropdownHandle;
        ValueHandle<CuboidVolume> _cuboidVolumeHandle;
        ValueHandle<SphereVolume> _sphereVolumeHandle;
        ValueHandle<CylinderVolume> _cylinderVolumeHandle;

        //NOTE: for handles it might be worth to make a dictionary where the 
        //      handle is a key to a list of the widgets using said handle.
        //      That way we can, based on changed handle update only widgets
        //      that are affected by a change in the handle.
        //  Change interface to PushChanges(IValueHandle handle)
        //      with that and the dictionary, you could simply do a loop over
        //      the value in the dictionary with 'handle' as key and redraw.

        CuboidWidget3D _cuboidVolumeWidget;
        SphereWidget3D _sphereVolumeWidget;
        CylinderWidget3D _cylinderVolumeWidget;

        DropdownWidget2D _volumeModeDropdown;
        protected override void BuildUI_3D()
        {


            /*
             Editor Responsibilities: 
              - Build the UI / Destroy the UI
              - Tie the values from the UI to the values of the condition
             */

            CreateWidget3D(
                widget:       out _cuboidVolumeWidget, 
                valueHandle: out _cuboidVolumeHandle, 
                initialValue: Condition._volume as CuboidVolume
            );
            CreateWidget3D(
                widget:       out _sphereVolumeWidget, 
                valueHandle: out _sphereVolumeHandle,
                initialValue: Condition._volume as SphereVolume
            );
            CreateWidget3D(
                widget:       out _cylinderVolumeWidget, 
                valueHandle: out _cylinderVolumeHandle,
                initialValue: Condition._volume as CylinderVolume
            );
        }

        protected override void BuildUI_2D()
        {
            
            CreateWidget2D(
                widget:       out _volumeModeDropdown, 
                valueHandle: out _dropdownHandle, 
                initialValue: (int)Condition.volumeMode, 
                setup:        w => w.SetOptions(Condition.volumeMode) 
           );
        }

        protected override void DestroyUI_3D()
        {
            //TODO: leave visual representations of the conditions
            Destroy(_cuboidVolumeWidget.gameObject);        // Destroy the component, the component then destroys the object.
            Destroy(_sphereVolumeWidget.gameObject);        // Destroy(cuboidVolumeWidget.gameobject) does not work here.
            Destroy(_cylinderVolumeWidget.gameObject);
        }
        protected override void DestroyUI_2D()
        {
            Destroy(_volumeModeDropdown.gameObject);
        }

        public override void OnPushChanges()
        {
            //just testing stuff
            //editorMode = modeDropdown.GetEnumValue<EditorMode>();
            //switch ((EditorMode)_dropdownHandle.Value)
            //{
            //    case EditorMode.Sphere:
            //        cuboidVolumeWidget.gameObject.SetActive(false);

            //        //Debug.Log("Sphere!");
            //        break;
            //    case EditorMode.Cuboid:
            //        //Debug.Log("Cuboid!");
            //        break;
            //    case EditorMode.Cyllinder:
            //        //Debug.Log("Cyllinder!");
            //        break;
            //}
            switch ((PositionCondition.VolumeMode)_dropdownHandle.Value)
            {
                case PositionCondition.VolumeMode.Cuboid:
                    _cuboidVolumeWidget.gameObject.SetActive(true);
                    _sphereVolumeWidget.gameObject.SetActive(false);
                    _cylinderVolumeWidget.gameObject.SetActive(false);
                    Condition._volume = _cuboidVolumeHandle.Value;
                break;
                case PositionCondition.VolumeMode.Sphere:
                    _sphereVolumeWidget.gameObject.SetActive(true);
                    _cuboidVolumeWidget.gameObject.SetActive(false);
                    _cylinderVolumeWidget.gameObject.SetActive(false);
                    Condition._volume = _sphereVolumeHandle.Value;
                break;
                case PositionCondition.VolumeMode.Cylinder:
                    _cylinderVolumeWidget.gameObject.SetActive(true);
                    _sphereVolumeWidget.gameObject.SetActive(false);
                    _cuboidVolumeWidget.gameObject.SetActive(false);
                    Condition._volume = _cylinderVolumeHandle.Value;
                break;
                default:
                    throw new System.NotImplementedException("No widgets avalable to get values from!");
            }
            //this needs to be persistent between selecting different conditions
            Condition.volumeMode = (PositionCondition.VolumeMode)_dropdownHandle.Value;
        }
    }
}
