﻿using UnityEngine;
using UnityEngine.UI;

namespace Playground.UI
{
    /// <summary>
    /// Manages the 2D Widgets of a ConditionEditor
    /// </summary>
    public class ConditionEditorMenu : MonoBehaviour
    {

        private const string BUTTON_LABEL_RESET = "Place Condition Widget by Relative";
        private const string BUTTON_LABEL_RESET_UNSET = "Place Condition Widget by Relative";
#pragma warning disable 0649
        [SerializeField] private Text _txtConditionLabel;
        [SerializeField] private Text _txtRelativeButtonLabel;

        [SerializeField] private Button _btnSetRelative;
        [SerializeField] private Button _btnResetWidgetByRelative;
        [SerializeField] private RectTransform _widgetContainer;
#pragma warning restore 0649

        public IConditionEditor currentEditor { get; private set; }
        // from the instantiation point of the condition,
        // pass the new editor in, and clear the widgets
        // (i.e. destroy the conditionEditor)

        private float nextPlacementPosition;
        internal void AddWidget( Widget2D widget )
        {
            widget.transform.SetParent(_widgetContainer, false);

            //TODO: Meke the widgets place correctly.

            widget.RectTransform.sizeDelta = new Vector2(_widgetContainer.rect.width, widget.RectTransform.rect.height);
            widget.transform.localScale = Vector3.one;
            widget.transform.localRotation = Quaternion.identity;
            //widget.transform.localPosition = new Vector3(0, nextPlacementPosition);
            widget.RectTransform.anchoredPosition = new Vector2(0, nextPlacementPosition);
            nextPlacementPosition += widget.RectTransform.rect.height;
        }

        internal void Clear()
        {
            currentEditor?.DestroyEditor();
            //NOTE: Apparently an interface is capable of calling methods on the
            //      destroyed underlying implementee, even after it's destroyed.
            //      - Set it to null to prevent this.
            currentEditor = null;
            _txtConditionLabel.text = "[Condition]";
            _btnSetRelative.gameObject.SetActive(false);
            _txtRelativeButtonLabel.gameObject.SetActive(false);
            _btnResetWidgetByRelative.gameObject.SetActive(false);
        }

        internal void SetTarget(Condition condition, string label)
        {
            if (currentEditor?.GetCondition() == condition)
                return; //we're already editing this condition, no point in recreating it!

            _btnSetRelative.gameObject.SetActive(true);
            _txtRelativeButtonLabel.gameObject.SetActive(true);
            _btnResetWidgetByRelative.gameObject.SetActive(true);

            nextPlacementPosition = 0;

            var conditionEditor = Internal.ConditionTypes.CreateConditionEditorObject(condition);

            currentEditor?.DestroyEditor();
            currentEditor = conditionEditor;

            _txtConditionLabel.text = label;
            if(condition is Internal.IHasRelative conditionWithRelative)
            {
                _btnSetRelative.onClick.RemoveAllListeners();
                _btnSetRelative.onClick.AddListener(() => { //TODO add stronger indication on selection mode
                    GameManager.Instance.laserPointer.SetSelectionMode( LaserPointer.SelectionMode.Relative);
                    _txtRelativeButtonLabel.text = "Cancel";
                    //set button to 'unclick' mode
                    _btnSetRelative.onClick.AddListener(() => {
                        GameManager.Instance.laserPointer.SetSelectionMode(LaserPointer.SelectionMode.Context);
                        _txtRelativeButtonLabel.text = $"{conditionWithRelative.Relative.name} <{conditionWithRelative.RelativeType.Name}>";
                    });
                });
                _btnSetRelative.interactable = true;
                _txtRelativeButtonLabel.text = conditionWithRelative.Relative?.name ??  $"[World <{conditionWithRelative.RelativeType.Name}>]";
            }
            else
            {
                _btnSetRelative.interactable = false;
                _txtRelativeButtonLabel.text = "[Nothing]";
            }
            foreach (var widget2D in currentEditor.Widget2Ds) {
                AddWidget(widget2D);
            }
        }

        internal void SetRelative( SceneObject selectedObject )
        {
            if (currentEditor.GetCondition() is Internal.IHasRelative conditionWithRelative)
                SetRelative(selectedObject, conditionWithRelative);
        }

        internal void SetRelative(SceneObject selectedObject, Internal.IHasRelative conditionWithRelative)
        {
            conditionWithRelative.SetRelative(selectedObject.GetComponent(conditionWithRelative.RelativeType));
            selectedObject.OnSetRelative();
            _txtRelativeButtonLabel.text = $"{conditionWithRelative.Relative.name} <{conditionWithRelative.RelativeType.Name}>";
            _btnResetWidgetByRelative.onClick.RemoveAllListeners();
            _btnResetWidgetByRelative.onClick.AddListener(() => { //TODO: Make this cancelable
                foreach(var widget in currentEditor.Widget3Ds) {
                    widget.SetInitialState();
                }
                foreach (var widget in currentEditor.Widget2Ds) {
                    widget.SetInitialState();
                }
            });
            foreach (var widget3D in currentEditor.Widget3Ds)
                widget3D.transform.SetParent(conditionWithRelative.Relative.transform, true);

            GameManager.Instance.laserPointer.SetSelectionMode(LaserPointer.SelectionMode.Context);
        }

        internal void SetWorldRelative( Internal.IHasRelative conditionWithRelative )
        {
            _txtRelativeButtonLabel.text = $"[World <{conditionWithRelative.RelativeType.Name}>]";
            _btnResetWidgetByRelative.onClick.RemoveAllListeners();
            _btnResetWidgetByRelative.onClick.AddListener(() => { //TODO: Make this cancelable
                foreach (var widget in currentEditor.Widget3Ds)
                {
                    widget.SetInitialState();
                }
                foreach (var widget in currentEditor.Widget2Ds)
                {
                    widget.SetInitialState();
                }
            });
            foreach (var widget3D in currentEditor.Widget3Ds)
                widget3D.transform.SetParent(null, true);

            GameManager.Instance.laserPointer.SetSelectionMode(LaserPointer.SelectionMode.Context);
        }
    }
}
