﻿using System.Collections.Generic;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

namespace Playground.UI
{
    public abstract class MenuItem : Button
    {
        [SerializeField] private Text _label;
        public Text Label
        {
            get => _label;
            set => _label = value;
        }

        protected Transform _parent;
        protected int siblingIndex;
        public new RectTransform transform => base.transform as RectTransform;

    }
    [System.Serializable]
    public abstract class MenuItem<T> : MenuItem, IDragHandler, IBeginDragHandler, IEndDragHandler where T : class
    {
        protected T _referredItem;
        public T RefferedItem => _referredItem;
        
        public IMenuItemContainer<T> container;

        public void SetRefferedItem(T item)
        {
            _referredItem = item;
            //TODO: set name here too? using ToString, or add an interface INamed that requires a getter for string Name
            //Label = item.ToString(); -> i.e. make sure to override ToString on things like Condition
        }

        protected override void Start()
        {
            base.Start();

            _parent = transform.parent;
        }

        public virtual void OnBeginDrag(PointerEventData eventData)
        {
            if (eventData.pointerDrag.transform.parent != GameManager.Instance.Canvas.transform)
            {
                _parent = eventData.pointerDrag.transform.parent;
                siblingIndex = transform.GetSiblingIndex();
            }
            
            transform.SetParent(GameManager.Instance.Canvas.transform, true);
            transform.GetComponent<CanvasGroup>().blocksRaycasts = false;

            //eventData.Use();
            //QOL: Set some color or something to indicate it's beeing dragged
            //var parentScale = transform.parent.lossyScale;
            //transform.localScale = new Vector3(1 / parentScale.x, 1 / parentScale.y, 1 / parentScale.z); 
        }

        public virtual void OnDrag( PointerEventData eventData )
        {
            eventData.selectedObject.transform.position = eventData.pointerCurrentRaycast.worldPosition;
        }

        public virtual void OnEndDrag(PointerEventData eventData)
        {
            Debug.Log($"Reverting Parent to: {_parent.name}");
            transform.SetParent(_parent, true);
            transform.SetSiblingIndex(siblingIndex);
            eventData.selectedObject = gameObject;
            List<RaycastResult> raycastResults = new List<RaycastResult>();
            EventSystem.current.RaycastAll(eventData, raycastResults);
            for (int i = 0; i < raycastResults.Count && !eventData.used; i++)
            {
                //Debug.Log(raycastResults[i].gameObject.name);
                var dropReciever = raycastResults[i].gameObject.GetComponent<IMenuItemDropReciever<T>>();
                if (dropReciever != null)
                {
                    dropReciever.OnRecievedItemDrop(this);
                    eventData.Use();
                }
            }
            if(!eventData.used) 
            {
                eventData.Use();
                DefaultDrop();
            }
            transform.GetComponent<CanvasGroup>().blocksRaycasts = true;
        }

        /// <summary>
        /// Fallback to drop the item back into the container in which it is already held.
        /// </summary>
        public virtual void DefaultDrop() => container.OnRecievedItemDrop(this);
    }
}