﻿using UnityEngine;
using UnityEngine.UI;

namespace Playground.UI
{
    [RequireComponent(typeof(Dropdown)), AddComponentMenu("Playground/Widget 2D/Dropdown")]
    public class DropdownWidget2D : Widget2D<int>
    {
#if UNITY_EDITOR
        protected override void Reset()
        {
            base.Reset();
            dropdown = GetComponent<Dropdown>();
        }
#endif
        public Dropdown dropdown;

        private void Awake()
        {
            //inject calls to our interface
            dropdown.onValueChanged.AddListener(_ => PropogatePushChanges()); //call directly to the base in order to propogate the event
        }
        /// <inheritdoc/>
        public override void SetInitialState() => dropdown.value = Value;
        protected override void OnConditionContextSet() => dropdown.value = Value;
        public void SetOptions( in System.Enum enumValue ) => SetOptions(System.Enum.GetNames(enumValue.GetType()));

        public void SetOptions(in string[] options)
        {
            dropdown.ClearOptions();
            dropdown.AddOptions(new System.Collections.Generic.List<string>(options));
        }
        public void SetOptions(in System.Collections.Generic.List<string> options)
        {
            dropdown.ClearOptions();
            dropdown.AddOptions(options);
        }

        public ref readonly T GetValue<T>( in T[] options ) => ref options[Value];

        public override void OnPushChanges() => Value = dropdown.value;
    }
}