﻿using UnityEngine;
namespace Playground.UI
{
    /// <summary>
    /// A widget defining a point in world space
    /// </summary>
    [SelectionBase, RequireComponent(typeof(LineRenderer)), AddComponentMenu("Playground/Widget 3D/Point")]
    public class PointWidget3D : WidgetVR<Vector3>
    {
#pragma warning disable 0649 //'_variable' is never assigned to, and will always have its default value null
        [SerializeField] private Material _active, _inactive;
        [SerializeField] private LineRenderer _lineRenderer;
        [SerializeField] private Renderer _renderer;
#pragma warning restore 0649

#if UNITY_EDITOR
        /* Only compiled when in unity editor, allows us to automatically 
         * get the wanted references upon adding/reseting the component
         */
        protected override void Reset()
        {
            base.Reset();
            _lineRenderer = GetComponent<LineRenderer>();
            _renderer = transform.GetChild(0).GetComponent<MeshRenderer>();
        }
#endif

        protected override void OnConditionContextSet()
        {
            transform.position = Condition.Context.transform.position;
        }

        public override void OnStartUpdate()
        {
            _renderer.sharedMaterial = _active;
            _lineRenderer.enabled = true;
            _lineRenderer.SetPosition(0, transform.position);
            _lineRenderer.SetPosition(1, transform.position);
        }
        public override void OnUpdateVisuals()
        {
            Value = transform.position;
            _lineRenderer.SetPosition(1, transform.position);
        }

        public override void OnStopUpdate()
        {
            _renderer.sharedMaterial = _inactive;
            _lineRenderer.enabled = false;
        }

        public override void OnPushChanges()
        {
            Value = transform.position;
        }

        public override void SetInitialState()
        {
            throw new System.NotImplementedException();
        }
    }
}
