﻿using UnityEngine;

namespace Playground.UI
{
    [SelectionBase, AddComponentMenu("Playground/Widget 3D/Scale Axis")]
    public class ScaleAxisWidget3D : WidgetVR<float>
    {
        [SerializeField] private Vector3 _axis = Vector3.right;
        [SerializeField] private Transform _collisionAndVisual;


        public Vector3 Axis => _owner?.transform.TransformDirection(_axis) ?? _axis;
        public Vector3 LocalAxis => _axis;

#if UNITY_EDITOR
        private void OnValidate()
        {
            _axis = new Vector3(Mathf.Abs(_axis.x), Mathf.Abs(_axis.y), Mathf.Abs(_axis.z)).normalized;
        }
        protected override void Reset()
        {
            base.Reset();
            _collisionAndVisual = transform.GetChild(0);
        }
#endif
        Vector3 startPosition;
        float initialOffset;
        Quaternion initialRotation = Quaternion.identity; //TODO: use this


        public override void SetInitialState()
        {
            //TODO
            throw new System.NotImplementedException();
        }

        public override void OnStartUpdate()
        {
            initialRotation = _collisionAndVisual.localRotation;
            initialOffset = Value;
            startPosition = transform.position;
        }

        public override void OnUpdateVisuals()
        {
            _collisionAndVisual.localRotation = initialRotation;
            Vector3 displacement = transform.position - startPosition;
            float moveDistance = Vector3.Dot(displacement, Axis);
            transform.position = startPosition + Axis * moveDistance;
            //QOL: Make the visuals not rotate
            Value = initialOffset + Vector3.Dot(displacement, Axis);
        }
        public override void OnStopUpdate()
        {
            _collisionAndVisual.localRotation = initialRotation;
            Vector3 displacement = transform.position - startPosition;
            float moveDistance = Vector3.Dot(displacement, Axis);
            transform.position = startPosition + Axis * moveDistance;
            transform.localRotation = Quaternion.identity;
            Value = initialOffset + Vector3.Dot(transform.position - startPosition, Axis);
        }

        public override void OnPushChanges() //DEPRECATED? : NO!
        {
            Value = initialOffset + Vector3.Dot(transform.position - startPosition, Axis);
        }

        private void OnDrawGizmosSelected()
        {
            Gizmos.color = new Color(_axis.x, _axis.y, _axis.z);
            Gizmos.DrawLine(transform.position - Axis, transform.position + Axis);
        }
    }
}