﻿using Playground.UI.Util;
using UnityEngine;
namespace Playground.UI
{
    /// <summary>
    /// Widget for defining a spherical volume.
    /// <para>Handles</para>
    /// <list type="bullet">
    /// <item>Position</item>
    /// <item>Radius</item>
    /// </list>
    /// </summary>
    [System.Serializable]
    public class SphereWidget3D : WidgetVR<SphereVolume>
    {
        [SerializeField] private PointWidget3D _rangePoint;
        [SerializeField] private Transform _meshTransform;
        [SerializeField] private LineRenderer _lineRenderer;

        
        /// <summary>
        /// Allow outside to control the Range point value handle
        /// </summary>
        /// <param name="handle">the new handle the widget should use</param>
        //TODO: There may actually be little to no use in having this.
        //      We're probablyt better off leaving the handle as it is
        //      And let the user access the value instead, through the handle.
        public void SetRangePointHandle( in ValueHandle<Vector3> handle ) => _rangePoint.SetValueHandle(handle);

#if UNITY_EDITOR
        /* Only compiled when in unity editor, allows us to automatically 
         * get the wanted references upon adding/reseting the component
         */
        protected override void Reset()
        {
            base.Reset();
            _rangePoint = transform.GetChild(1).GetComponent<PointWidget3D>();
            _meshTransform = transform.GetChild(0);
            _lineRenderer = GetComponent<LineRenderer>();
            _rangePoint._owner = this;
        }
#endif

        public override void SetInitialState()
        {
            WidgetUtil.PlaceInsideRelative(this);
            var radius = Condition.Context.transform.localScale.x * 0.5f;
            _meshTransform.localScale = Vector3.one * radius * 2.125f;
            _rangePoint.transform.position = transform.TransformPoint(new Vector3(_meshTransform.localScale.x * 0.5f, 0, 0));
        }

        /// <summary>
        /// Override to do component specific setup based on condition context.
        /// This is usually where you would ensure persistency by reconstructing
        /// the widget settings based on the value handle of the widget
        /// </summary>
        protected override void OnConditionContextSet()
        {
            if(Value == null)
            {
#if PLAYGROUND_DEBUG
                Debug.Log("WIDGET: Context condition uninitialized! Setting it up to default");
#endif
                SetInitialState();
            }
            else
            {
#if PLAYGROUND_DEBUG
                Debug.Log("WIDGET: Restoring Cuboid Widget");
#endif
                _rangePoint.transform.position = Value.radius * transform.right;
                transform.position = Value.center;
            }
        }

        public override void OnStartUpdate()
        {
            _lineRenderer.SetPosition(0, transform.position);
            _lineRenderer.SetPosition(1, transform.position);
            _lineRenderer.enabled = true;
        }
        public override void OnUpdateVisuals()
        {
            float d = Vector3.Distance(_rangePoint.transform.position, transform.position);
            _rangePoint.transform.localScale = new Vector3(d * 0.5f, d * 0.5f, d * 0.5f);
            _meshTransform.localScale = new Vector3(d * 2, d * 2, d * 2);
            _lineRenderer.SetPosition(1, _rangePoint.transform.position);
        }

        public override void OnStopUpdate()
        {
            float d = Vector3.Distance(_rangePoint.transform.position, transform.position);
            _rangePoint.transform.localScale = new Vector3(d * 0.5f, d * 0.5f, d * 0.5f);
            _meshTransform.localScale = new Vector3(d * 2, d * 2, d * 2);
            _lineRenderer.enabled = false;
        }
        public override void OnPushChanges()
        {
            float d = Vector3.Distance(_rangePoint.transform.position, transform.position);
            Value = new SphereVolume(transform.position, d);
        }
    }
}