﻿using UnityEngine;

namespace Playground.UI
{

    [System.Serializable] public class WidgetDictionary : LabeledList<Widget> { }
    [System.Serializable]
    [CreateAssetMenu(fileName = "WidgetCollection", menuName = "Playground/UI/WidgetCollection")]
    public class WidgetCollection : ScriptableObject
    {
        static WidgetCollection _instance;
        public static WidgetCollection Instance
        {
            get
            {
                if (!_instance)
                    _instance = Resources.Load<WidgetCollection>("WidgetCollection"); //TODO should we instead use FindAllOfType? - what about new collections? Singleton-issues?
                return _instance;
            }
        }

        [SerializeField] private WidgetDictionary _widgets = new WidgetDictionary();
        public Widget this[string str] => _widgets[str];

        public int WidgetCount => _widgets.Count;
        public void Add(string key, Widget widget)
        {
            if(_widgets.TryGetValue(key, out var existingWidget))
            {
                Debug.LogWarning("Widget with that key already exists!", existingWidget);
                _widgets.Add($"widget_{WidgetCount}", widget);
                return;
            }
            _widgets.Add(key, widget);
        }
        public void Remove(string key) => _widgets.Remove(key);
    }
}