﻿
using NUnit.Framework;

namespace Playground.Tests
{
    /// <summary>
    /// Mock-Condition: always returns true
    /// </summary>
    public class TrueCondition : Condition { public override bool Evaluate() => true; }
    
    /// <summary>
    /// Mock-Condition: always returns false
    /// </summary>
    public class FalseCondition : Condition { public override bool Evaluate() => false; }
    
    /// <summary>
    /// ============================================================
    /// <para>
    /// ConditionGroup_AND Tests
    /// </para>
    /// ============================================================
    /// </summary>
    public class ConditionGroup_AND_Test
    {
        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_AND))]
        public void ConditionGroup_AND_AllTrue()
        {
            var group = new ConditionGroup_AND();
            group.AddCondition(new TrueCondition());
            group.AddCondition(new TrueCondition());
            group.AddCondition(new TrueCondition());

            Assert.True(group.Evaluate(), "All conditions within evaluate to true, this should be true");
        }

        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_AND))]
        public void ConditionGroup_AND_AllFalse()
        {
            var group = new ConditionGroup_AND();
            group.AddCondition(new FalseCondition());
            group.AddCondition(new FalseCondition());
            group.AddCondition(new FalseCondition());

            Assert.False(group.Evaluate(), "All conditions within evaluate to false, this should be false");
        }

        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_AND))]
        public void ConditionGroup_AND_Mixed()
        {
            var group = new ConditionGroup_AND();
            group.AddCondition(new FalseCondition());
            group.AddCondition(new TrueCondition());
            group.AddCondition(new FalseCondition());

            Assert.False(group.Evaluate(), "Some conditions within evaluate to false, this should be false");
        }

        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_AND))]
        public void ConditionGroup_AND_NoConditionsWithin()
        {
            var group = new ConditionGroup_AND();
            Assert.True(group.Evaluate(), "No conditions within, this should be true");
        }
    }
    /// <summary>
    /// ============================================================
    /// <para>
    /// ConditionGroup_OR Tests
    /// </para>
    /// ============================================================
    /// </summary>
    public class ConditionGroup_OR_Test
    {
        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_OR))]
        public void ConditionGroup_OR_AllTrue()
        {
            var group = new ConditionGroup_OR();
            group.AddCondition(new TrueCondition());
            group.AddCondition(new TrueCondition());
            group.AddCondition(new TrueCondition());

            Assert.True(group.Evaluate(), "All conditions within evaluate to true, this should be true");
        }

        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_OR))]
        public void ConditionGroup_OR_AllFalse()
        {
            var group = new ConditionGroup_OR();
            group.AddCondition(new FalseCondition());
            group.AddCondition(new FalseCondition());
            group.AddCondition(new FalseCondition());

            Assert.False(group.Evaluate(), "All conditions within evaluate to false, this should be false");
        }

        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_OR))]
        public void ConditionGroup_OR_Mixed()
        {
            var group = new ConditionGroup_OR();
            group.AddCondition(new FalseCondition());
            group.AddCondition(new TrueCondition());
            group.AddCondition(new FalseCondition());

            Assert.True(group.Evaluate(), "Some conditions within evaluate to false, this should be true");
        }

        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_OR))]
        public void ConditionGroup_OR_NoConditionsWithin()
        {
            var group = new ConditionGroup_OR();
            Assert.True(group.Evaluate(), "No conditions within, this should be true");
        }
    }
    /// <summary>
    /// ============================================================
    /// <para>
    /// ConditionGroup_XOR Tests
    /// </para>
    /// ============================================================
    /// </summary>
    public class ConditionGroup_XOR_Test
    {
        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_XOR))]
        public void ConditionGroup_XOR_AllTrue()
        {
            var group = new ConditionGroup_XOR();
            group.AddCondition(new TrueCondition());
            group.AddCondition(new TrueCondition());
            group.AddCondition(new TrueCondition());

            Assert.False(group.Evaluate(), "All conditions within evaluate to true, this should be false");
        }


        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_XOR))]
        public void ConditionGroup_XOR_AllFalse()
        {
            var group = new ConditionGroup_XOR();
            group.AddCondition(new FalseCondition());
            group.AddCondition(new FalseCondition());
            group.AddCondition(new FalseCondition());

            Assert.False(group.Evaluate(), "All conditions within evaluate to false, this should be false");
        }


        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_XOR))]
        public void ConditionGroup_XOR_OnlyOneTrue()
        {
            var group = new ConditionGroup_XOR();
            group.AddCondition(new FalseCondition());
            group.AddCondition(new TrueCondition());
            group.AddCondition(new FalseCondition());

            Assert.True(group.Evaluate(), "One and only one condition within evaluate to true, this should be true");
        }


        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_XOR))]
        public void ConditionGroup_XOR_MoreThanOneTrue()
        {
            var group = new ConditionGroup_XOR();
            group.AddCondition(new FalseCondition());
            group.AddCondition(new TrueCondition());
            group.AddCondition(new TrueCondition());

            Assert.False(group.Evaluate(), "More than one condition within evaluate to true, this should be false");
        }


        [Test(Author = "Halvor", TestOf = typeof(ConditionGroup_XOR))]
        public void ConditionGroup_XOR_NoConditionsWithin()
        {
            var group = new ConditionGroup_XOR();
            Assert.True(group.Evaluate(), "No conditions within, this should be true? idk by def of the logic gate...");
        }
    }
}
