﻿using NUnit.Framework;
using UnityEngine;
using UnityEngine.TestTools;

namespace Playground.Tests
{
    // TODO: maybe split into one class each condition type to avoid clutter and also allow unique setup each condition?

    /// <inheritdoc cref="IMonoBehaviourTest" />
    /// <summary>
    /// Base class for testing conditions. Contains some useful setup and gameobjects to conduct test on
    /// </summary>
    public abstract class ConditionsTest : MonoBehaviour, IMonoBehaviourTest
    {
        private int testsCompleted;
        protected int testCount;
        public bool IsTestFinished => testsCompleted < testCount;   

        protected static TestData testData;
        
        [SetUp]
        public virtual void Setup()
        {
            testData = TestUtility.SetupDefaultScene();
        }   

        [TearDown]
        public virtual void Teardown()
        {
            if(testCount <= 0)
            {
                Debug.LogError(string.Format("invalid testCount: {0}", testCount), this);
            }

            Destroy(testData.contextObject.gameObject);
            if(testData.relativeObject)
                Destroy(testData.relativeObject.gameObject);

            testsCompleted++;
        }

    }

}

