﻿using System.Collections;
using UnityEngine;
using NUnit.Framework;
using UnityEngine.TestTools;

namespace Playground.Tests
{
    [TestOf(typeof(PositionCondition))]
    public class PositionConditionTest : ConditionsTest
    {
        static PositionCondition condition;

        [SetUp]
        public override void Setup()
        {
            testCount = 9;

            base.Setup();

            condition = testData.contextObject.gameObject.AddComponent<PositionCondition>();
            condition.Initialize(testData.contextObject);
            testData.contextObject.GoalConditions.AddCondition(condition);


        }
        //TODO: These shouldnt have to be unityTests, as they have no need for more than one frame D:
        //However, because of how it's structured atm, they need to be so :(
        [UnityTest] 
        public IEnumerator CuboidVolumeTest_NoRelative_Inside()
        {
            yield return null;
            var volume = new CuboidVolume(
                new Vector3(0f, 0f, 0f),
                new Vector3(1f, 1f, 1f),
                Matrix4x4.identity
            );
            condition._volume = volume;
            if (testData.relativeObject)
                Destroy(testData.relativeObject.gameObject); //in case there's a relative object left over
            testData.contextObject.transform.position = new Vector3(0.5f, 0.5f, 0.5f);
            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator CuboidVolumeTest_NoRelative_Inside_Edge()
        {
            yield return null;
            var volume = new CuboidVolume(
                new Vector3(-10f, -4f, -2f),
                new Vector3(5f, 2f, 3f),
                Matrix4x4.identity
            );
            condition._volume = volume;
            if (testData.relativeObject)
                Destroy(testData.relativeObject.gameObject); //in case there's a relative object left over
            testData.contextObject.transform.position = new Vector3(-10f, -4f, -2f);
            var evaluation = TestUtility.EvaluateConditions(testData.contextObject);
            Assert.IsTrue(evaluation);
        }

        [UnityTest]
        public IEnumerator CuboidVolumeTest_NoRelative_Outside()
        {
            yield return null;
            var volume = new CuboidVolume(
                new Vector3(0f, 0f, 0f),
                new Vector3(1f, 1f, 1f),
                Matrix4x4.identity
            );
            condition._volume = volume;
            if (testData.relativeObject)
                Destroy(testData.relativeObject.gameObject); //in case there's a relative object left over
            testData.contextObject.transform.position = new Vector3(-0.5f, 0.5f, 0.5f);
            Assert.IsFalse(TestUtility.EvaluateConditions(testData.contextObject));
        }


        [UnityTest]
        public IEnumerator CuboidVolumeTest_Relative_Inside()
        {
            yield return null;
            condition.SetRelative(testData.relativeObject.transform);
            testData.relativeObject.transform.position = new Vector3(2, 2, 2);
            //testData.relativeObject.transform.rotation = new Quaternion(1, 0, 0, 2);
            Vector3 volumePosition = testData.relativeObject.transform.position + testData.relativeObject.transform.up * 2;
            var volume = new CuboidVolume(
                new Vector3(-10f, -4f, -2f),
                new Vector3(5f, 2f, 3f),
                Matrix4x4.TRS(volumePosition, Quaternion.identity, Vector3.one).inverse
            );
            condition._volume = volume;
            testData.contextObject.transform.position = volumePosition + (volume.min + volume.max)*0.5f;
            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator CuboidVolumeTest_Relative_Inside_Edge()
        {
            yield return null;
            condition.SetRelative(testData.relativeObject.transform);
            testData.relativeObject.transform.position = new Vector3(2, 2, 2);
            //testData.relativeObject.transform.rotation = new Quaternion(1, 0, 0, 2);
            Vector3 volumePosition = testData.relativeObject.transform.position + testData.relativeObject.transform.up * 2;
            var volume = new CuboidVolume(
                new Vector3(-10f, -4f, -2f),
                new Vector3(5f, 2f, 3f),
                Matrix4x4.TRS(volumePosition, Quaternion.identity, Vector3.one).inverse
            );
            condition._volume = volume;
            testData.contextObject.transform.position = volumePosition + volume.min;
            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator CuboidVolumeTest_Relative_Outside()
        {
            yield return null;
            condition.SetRelative(testData.relativeObject.transform);
            testData.relativeObject.transform.position = new Vector3(2, 2, 2);
            //testData.relativeObject.transform.rotation = new Quaternion(1, 0, 0, 2);
            Vector3 volumePosition = testData.relativeObject.transform.position + Vector3.up * 2;
            var volume = new CuboidVolume(new Vector3(-10f, -4f, -2f), new Vector3(5f, 2f, 3f),
                Matrix4x4.TRS(volumePosition, Quaternion.identity, Vector3.one).inverse);

            condition._volume = volume;
            testData.contextObject.transform.position = volumePosition + volume.max + new Vector3(0,10,0);
            Assert.IsFalse(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator SphereVolumeTest_NoRelative_Inside()
        {
            yield return null;
            var volume = new SphereVolume(new Vector3(-10f, -4f, -2f), 3f);
            condition._volume = volume;
            testData.contextObject.transform.position = volume.center;
            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator SphereVolumeTest_NoRelative_Inside_Edge()
        {
            yield return null;
            var volume = new SphereVolume(new Vector3(-10f, -4f, -2f), 3f);

            condition._volume = volume;
            if (testData.relativeObject)
                Destroy(testData.relativeObject.gameObject); //in case there's a relative object left over
            testData.contextObject.transform.position = volume.center + Vector3.right * volume.radius;
            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator SphereVolumeTest_NoRelative_Outside()
        {
            yield return null;
            var volume = new SphereVolume(new Vector3(-10f, -4f, -2f), 3f);
            condition._volume = volume;
            if (testData.relativeObject)
                Destroy(testData.relativeObject.gameObject); //in case there's a relative object left over
            testData.contextObject.transform.position = volume.center + Vector3.right * (volume.radius + 100);
            Assert.IsFalse(TestUtility.EvaluateConditions(testData.contextObject));
        }

    }
}

