﻿using System.Collections;
using UnityEngine;
using UnityEngine.TestTools;
using NUnit.Framework;

namespace Playground.Tests
{
    /// <summary>
    /// Class for testing touch condition
    /// </summary>
    public class TouchConditionTest : ConditionsTest
    {    
        [SetUp]
        public override void Setup()
        {
            testCount = 2;

            base.Setup();

            TouchCondition condition = testData.contextObject.gameObject.AddComponent<TouchCondition>();

            condition.Initialize(testData.contextObject);
            condition.SetRelative(testData.relativeObject.GetComponent<Collider>());

            testData.contextObject.GoalConditions.AddCondition(condition);
        }

        [UnityTest]
        public IEnumerator NoContact_WithRelative_ResultsToFalse()
        {
            testData.contextObject.transform.position = new Vector3(0, 200, 0);
            yield return new WaitForFixedUpdate();//wait for physics

            Assert.IsFalse(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator Touching_Relative_ResultsToTrue()
        {
            testData.contextObject.transform.position = new Vector3(0, 0.9f, 0);
            for (int i = 0; i < 30; i++)
                yield return new WaitForFixedUpdate();

            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));
        }

        // teardown in base
    }
}

