﻿using NUnit.Framework;
using System.Collections;
using UnityEngine;
using UnityEngine.TestTools;

namespace Playground.Tests
{
    [System.Serializable]
    public class VelocityConditionTest : ConditionsTest
    {
        static VelocityCondition condition;


        [SetUp]
        public override void Setup()
        {
            testCount = 4;

            base.Setup();

            condition = testData.contextObject.gameObject.AddComponent<VelocityCondition>();
            condition.Initialize(testData.contextObject);
            testData.contextObject.GoalConditions.AddCondition(condition);

            condition.MaxVelocity = Mathf.Infinity;
            condition.MinVelocity = Mathf.NegativeInfinity;
        }

        [UnityTest]
        public IEnumerator LessVelocity_ThanMax_ResultToTrue()
        {
            condition.MaxVelocity = 100f;

            for (int i = 0; i < 10; i++)
                yield return new WaitForFixedUpdate();//wait for physics

            testData.contextObject.GetComponent<Rigidbody>().velocity = new Vector3(0, 0, 0);
            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));

        }

        [UnityTest]
        public IEnumerator GreaterVelocity_ThanMax_ResultToFalse()
        {
            condition.MaxVelocity = 100f;
            for (int i = 0; i < 10; i++)
                yield return new WaitForFixedUpdate();

            testData.contextObject.GetComponent<Rigidbody>().velocity = new Vector3(1000f, 1000f, 1000f);
            Assert.IsFalse(TestUtility.EvaluateConditions(testData.contextObject));

        }

        [UnityTest]
        public IEnumerator LessVelocity_ThanMin_ResultToFalse()
        {
            condition.MinVelocity = 17f;
            for (int i = 0; i < 10; i++)
                yield return new WaitForFixedUpdate();

            testData.contextObject.GetComponent<Rigidbody>().velocity = new Vector3(9.5f, 9.5f, 9.5f); // 16.x velocity magnitude
            Assert.IsFalse(TestUtility.EvaluateConditions(testData.contextObject));

        }

        [UnityTest]
        public IEnumerator GreaterVelocity_ThanMin_ResultToTrue()
        {
            condition.MinVelocity = 10f;

            for (int i = 0; i < 10; i++)
                yield return new WaitForFixedUpdate();

            testData.contextObject.GetComponent<Rigidbody>().velocity = new Vector3(9.5f, 9.5f, 9.5f);
            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));

        }


    }
}

