﻿using NUnit.Framework;
using System.Collections;
using UnityEngine;
using UnityEngine.TestTools;

namespace Playground.Tests
{
    public class VolumeConditionTest : ConditionsTest
    {
        static VolumeCondition condition;
        static GameObject volumeObject;
        static Collider volumeCollider;

        [SetUp]
        public override void Setup()
        {
            testCount = 3;

            base.Setup();

            volumeObject = GameObject.CreatePrimitive(PrimitiveType.Cube);
            volumeCollider = volumeObject.AddComponent<BoxCollider>();
            volumeCollider.isTrigger = true;

            condition = testData.contextObject.gameObject.AddComponent<VolumeCondition>();
            condition.Initialize(testData.contextObject);
            condition.SetRelative(testData.relativeObject.GetComponent<Transform>());
            condition.SetVolume(volumeCollider);

            testData.contextObject.GoalConditions.AddCondition(condition);

            volumeObject.transform.position = Vector3.zero;
            testData.contextObject.transform.position = Vector3.zero;
            testData.contextObject.GetComponent<Rigidbody>().isKinematic = true;
        }

        [UnityTest]
        public IEnumerator Context_InsideVolume_ResultsToTrue()
        {
            volumeObject.transform.localScale *= 1.1f;

            for (int i = 0; i < 10; i++)
                yield return new WaitForFixedUpdate();

            Assert.IsTrue(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator Context_InsideButBiggerVolume_ResultsToFalse()
        {
            volumeObject.transform.localScale *= 0.8f;

            for (int i = 0; i < 10; i++)
                yield return new WaitForFixedUpdate();

            Assert.IsFalse(TestUtility.EvaluateConditions(testData.contextObject));
        }

        [UnityTest]
        public IEnumerator Context_PartiallyInsideVolume_ResultsToFalse()
        {
            volumeObject.transform.position = new Vector3(0.5f, 0, 0);

            for (int i = 0; i < 10; i++)
                yield return new WaitForFixedUpdate();

            Assert.IsFalse(TestUtility.EvaluateConditions(testData.contextObject));
        }

        // TODO: relative

        [TearDown]
        public override void Teardown()
        {
            Destroy(volumeObject);
            base.Teardown();
        }
    }
}

