#!/bin/sh


directory=`dirname $0`

# do linting on the code that is about to be pushed
remoteAndBranch=$(git rev-parse origin/master)
workingFiles=$(git diff --stat --no-commit-id --ignore-submodules --name-only -r  $remoteAndBranch..)

whitelist=[]
counter=0

while IFS='' read line || [[ -n "$line" ]]; do
	if [[ $line == "WHITELISTEND"* ]]
	then
		break
	fi
	whitelistinput="${line//$'\r'/}"
	whitelist[$counter]=$whitelistinput
	counter=$((counter+1))
done < "$directory/../../InternalDevTools/GitHooks/whitelist.txt"
# mapfile < $directory/../../InternalDevTools/GitHooks/whitelist.txt whitelist
for item in ${whitelist[@]}
do
	echo "Whitelisted member: " $item
done

# source: https://stackoverflow.com/a/638980
if [ ! -f $directory/../../playground.sln ]; then
	for file in $workingFiles
	do
		if [[ $file == *".cs"* ]]
		then
			echo "could not find playground.sln in project root \n scripts are modified, either get sln or revert script changes"
			exit 1
		fi
	done
fi

start=`date +%s`

globalResult=$directory/Resharper/Result/InspectionResult.xml
$directory/Resharper/inspectcode.exe --no-swea -s=WARNING $directory/../../playground.sln -o=$globalResult --project=Playground.*
atIssuesSection=false

personalFile=$directory/Resharper/Result/PersonalInspectionResult.txt

if [ -e $personalFile ]; then
    rm $personalFile
fi

touch $personalFile
linterFailed=false
workingFiles=$(echo  $workingFiles|tr / '\\')

while IFS='' read -r line || [[ -n "$line" ]]; do
   	#if we are done reading
	if [[ $line == *"TypeId"* ]]
	then
		for file in $workingFiles
		do
			failed=true
			if [[ $line == *"$file"* ]];
			then
				for item in ${whitelist[@]}
				do
					if [[ $line == *"$item"* ]]
					then
						failed=false
						break;
					fi
				done
				if [ $failed == true ]
				then
					linterFailed=true
					echo $line >> $personalFile
				fi
			fi
		done
	fi
done < $globalResult

end=`date +%s`
runtime=$((end-start))

echo "Linting process took " $runtime " seconds."
if [ $linterFailed == true ]
then
	echo "Linter failed. Cancelling push request."
	$(start $personalFile)
	exit 1
fi
echo "Linting was sucessfull, returning STATUS_OK(0) to pre-push"
exit 0
