# Neodroid Playground

## Summary
This project is the bachelor project at *NTNU Gjøvik* by:
- Halvor Bakken Smedås
- Nikolai Åkerholt
- Jone Martin Skaara
- Aksel Hjerpbakk

The bachelor task is with *SINTEF Ocean SeaLabs* located in Trondheim.

## Goal of project
To have a framework that can be used by staff in industries in many different disciplines to demonstrate how a given task should be performed, and then have a robot trained in this task during the following hours / days.
This project serves as a final abstraction layer for the end user, which allows the user to annotate the agent environment with minimal understanding of Unity or AI needed through this VR application. The goal is therefore to allow any layman to create a valid environment for an agent to operate in.

## Components
- **SceneObjects:**  
  A *GameObject* in the scene that has two lists of *conditions*, one for goal conditions, all of these need to be completed in order for the sceneObject to evaluate as fulfilled, and a terminating list in which only one need to be true for the object to evaluate as terminating.
- **Conditions:**  
  We put constraints on the *SceneObjects* in the scene that we call *conditions*, these evaluate to true when they are fulfilled. An example of this is *position conditions* that are fulfilled when the *SceneObject* is moved inside a volume defined by a *3D widget*. All conditions have a concept of a relative SceneObject, although they may implement it differently. In the case of the *position condition* the volume will follow the relative object. In the case of the *touch condition*, the relative determines which object it needs to be colliding with.
- **Logical groups:**  
  Conditions can be put into logical groups:
    - **AND group:** All *conditions* in the group must be fulfilled before the AND group is fulfilled.
    - **OR group:** Any of the *conditions* in the group can be fulfilled for the OR group is fulfilled.
    - **XOR group:** Exactly one of the *conditions* in the group must be fulfilled for the XOR group is fulfilled.
- **3D widget:**  
  A *GameObject* in the scene that is VR interactable, and is used to define a volume, a vector, numerical value or some other property that could be used as input to a *condition*.
- **2D widget:**  
  UI elements such as sliders and dropdown menus that are used to define input to *conditions* similarly to *3D widgets*. Although *2D widgets* are operated using a laser pointer in VR, or potentially a mouse outside VR.
- **Interface - hand tablet:**  
  The interface between the user and the *Neodroid Playground*. In VR pressing the menu button on the controllers bring up a floating menu in that has three panels:
  - **Status panel**: has a summary of the how many objects are in the scene and how many are in the goal and terminating states, as well as how any *conditions* are present. The status menu also has a few controls to reload the scene, wipe all *conditions*, save the preset of *conditions* to file so it can be loaded later, or starting the AI training.
  - **Condition overview panel:** is only available when a *SceneObject* is selected. It has three lists: a list of all the *condition* types that exist, a goal list of *conditions* for the *SceneObject*, and a terminating list of *conditions*. To add *conditions* to the selected object, drag one from the list into the desired list.
  - **Condition editor panel:** is only available when a *Condition* is selected within one of the two state lists in the *Condition overview panel*. In the condition editor are all the 2D widgets related to that *condition*.
- **Panda hand:**  
  The panda hand is our implementation of the hand on the *Franka Emika Panda* robot arm. It can be used to pick up objects in VR using physics in Unity. The goal of for this object is to act as realistic as possible. Colliding with walls in a realistic way, without deforming the fingers at the tip and in general behave like the motor powered robot arm is expected to move and behave in real life. A learning process will eventually use output form the *Neodroid Playground* to learn based on visual data. If the physics don't act realistically the leaning process won't be able to learn how to do that task in real life.

## Dependencies
- SteamVR #TODO More on dependencies and install instructions
- droid
- neo #TODO More on neo's dependencies and install instructions

## Install Instructions
Scene, SteamVR, Game Manager, Condition Data Manager, Prototyping environment, actor.
